@@ -1,26 +1,39 @@
 /*
- *  Copyright 2006-2012 Adrian Thurston <thurston@complang.org>
+ * Copyright 2006-2012 Adrian Thurston <thurston@colm.net>
+ *
+ * Permission is hereby granted, free of charge, to any person obtaining a copy
+ * of this software and associated documentation files (the "Software"), to
+ * deal in the Software without restriction, including without limitation the
+ * rights to use, copy, modify, merge, publish, distribute, sublicense, and/or
+ * sell copies of the Software, and to permit persons to whom the Software is
+ * furnished to do so, subject to the following conditions:
+ *
+ * The above copyright notice and this permission notice shall be included in all
+ * copies or substantial portions of the Software.
+ *
+ * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
+ * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
+ * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
+ * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
+ * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
+ * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
+ * SOFTWARE.
  */
 
-#include <iostream>
-#include <iomanip>
+#include "compiler.h"
+
 #include <errno.h>
 #include <stdlib.h>
-#include <limits.h>
-#include <sstream>
+#include <string.h>
+#include <stdbool.h>
+#include <unistd.h>
+#include <assert.h>
+#include <iostream>
 
-#include "global.h"
-#include "avltree.h"
-#include "compiler.h"
-#include "parser.h"
-#include "parsetree.h"
-#include "mergesort.h"
 #include "redbuild.h"
 #include "pdacodegen.h"
 #include "fsmcodegen.h"
-#include "pdarun.h"
 #include "colm.h"
-#include "pool.h"
 
 using std::ostringstream;
 using std::cout;
@@ -346,10 +359,7 @@ Compiler::Compiler( )
 	//tokenStruct(0),
 
 	ptrLangEl(0),
-	boolLangEl(0),
-	intLangEl(0),
 	strLangEl(0),
-	streamLangEl(0),
 	anyLangEl(0),
 	rootLangEl(0),
 	noTokenLangEl(0),
@@ -360,22 +370,27 @@ Compiler::Compiler( )
 	firstNonTermId(0),
 	prodIdIndex(0),
 
+	global(0),
+	globalSel(0),
 	globalObjectDef(0),
-	argv0(0),
-	argvList(0),
+	arg0(0),
+	argv(0),
+
+	stream(0),
+	streamSel(0),
 
 	uniqueTypeNil(0),
 	uniqueTypePtr(0),
 	uniqueTypeBool(0),
 	uniqueTypeInt(0),
 	uniqueTypeStr(0),
-	uniqueTypeStream(0),
 	uniqueTypeIgnore(0),
 	uniqueTypeAny(0),
+	uniqueTypeStream(0),
 	nextPatConsId(0),
 	nextGenericId(1),
 	nextFuncId(0),
-	loopCleanup(0),
+	nextHostId(0),
 	nextObjectId(1),     /* 0 is  reserved for no object. */
 	nextFrameId(0),
 	nextParserId(0),
@@ -395,6 +410,15 @@ Compiler::~Compiler()
 	/* Delete all the nodes in the action list. Will cause all the
 	 * string data that represents the actions to be deallocated. */
 	actionList.empty();
+
+	for ( CharVectVect::Iter fns = streamFileNames; fns.lte(); fns++ ) {
+		const char **ptr = *fns;
+		while ( *ptr != 0 ) {
+			::free( (void*)*ptr );
+			ptr += 1;
+		}
+		free( (void*) *fns );
+	}
 }
 
 ostream &operator<<( ostream &out, const Token &token )
@@ -743,11 +767,13 @@ LangEl *Compiler::makeRepeatProd( const InputLoc &loc, Namespace *nspace,
 
 	/* Build the first production of the repeat. */
 	TypeRef *typeRef1 = TypeRef::cons( loc, ut );
-	ProdEl *factor1 = new ProdEl( ProdEl::ReferenceType, InputLoc(), 0, false, typeRef1, 0 );
+	ProdEl *factor1 = new ProdEl( ProdEl::ReferenceType,
+			InputLoc(), 0, false, typeRef1, 0 );
 
 	UniqueType *prodNameUT = findUniqueType( TYPE_TREE, prodName );
 	TypeRef *typeRef2 = TypeRef::cons( loc, prodNameUT );
-	ProdEl *factor2 = new ProdEl( ProdEl::ReferenceType, InputLoc(), 0, false, typeRef2, 0 );
+	ProdEl *factor2 = new ProdEl( ProdEl::ReferenceType,
+			InputLoc(), 0, false, typeRef2, 0 );
 
 	prodElList1->append( factor1 );
 	prodElList1->append( factor2 );
@@ -856,6 +882,15 @@ Namespace *Namespace::findNamespace( const String &name )
 	return 0;
 }
 
+Reduction *Namespace::findReduction( const String &name )
+{
+	for ( ReductionVect::Iter r = reductions; r.lte(); r++ ) {
+		if ( strcmp( name, (*r)->name ) == 0 )
+			return *r;
+	}
+	return 0;
+}
+
 /* Search from a previously resolved qualification. (name 1+ in a qual list). */
 Namespace *NamespaceQual::searchFrom( Namespace *from, StringVect::Iter &qualPart )
 {
@@ -956,32 +991,29 @@ void Compiler::initEmptyScanners()
 	}
 }
 
-PdaRun *Compiler::parsePattern( Program *prg, Tree **sp, const InputLoc &loc, 
-		int parserId, StreamImpl *sourceStream )
+pda_run *Compiler::parsePattern( program_t *prg, tree_t **sp, const InputLoc &loc, 
+		int parserId, struct stream_impl *sourceStream )
 {
-	StreamImpl *in = newSourceStreamGeneric( "<internal>" );
+	struct stream_impl *in = colm_impl_new_generic( strdup("<internal>") );
 
-	PdaRun *pdaRun = new PdaRun;
-	initPdaRun( prg, pdaRun, pdaTables, parserId, 0, false, 0 );
+	struct pda_run *pdaRun = new pda_run;
+	colm_pda_init( prg, pdaRun, pdaTables, parserId, 0, false, 0, false );
 
-	Stream *res = streamAllocate( prg );
-	res->id = LEL_ID_STREAM;
-	res->in = sourceStream;
+	stream_t *stream = colm_stream_new_struct( prg );
+	stream->impl = sourceStream;
 
-	Tree *ptr = constructPointer( prg, (Tree*)res );
+	in->funcs->append_stream( in, (tree_t*)stream );
+	in->funcs->set_eof( in );
 
-	in->funcs->appendStream( in, ptr );
-	in->funcs->setEof( in );
-
-	long pcr = parseLoop( prg, sp, pdaRun, in, PcrStart );
-	assert( pcr == PcrDone );
-	if ( pdaRun->parseError ) {
+	long pcr = colm_parse_loop( prg, sp, pdaRun, in, PCR_START );
+	assert( pcr == PCR_DONE );
+	if ( pdaRun->parse_error ) {
 		cerr << ( loc.fileName != 0 ? loc.fileName : "<input>" ) <<
 				":" << loc.line << ":" << loc.col;
 
-		if ( pdaRun->parseErrorText != 0 ) {
+		if ( pdaRun->parse_error_text != 0 ) {
 			cerr << ": relative error: " << 
-					pdaRun->parseErrorText->tokdata->data;
+					pdaRun->parse_error_text->tokdata->data;
 		}
 		else {
 			cerr << ": parse error";
@@ -994,23 +1026,26 @@ PdaRun *Compiler::parsePattern( Program *prg, Tree **sp, const InputLoc &loc,
 	return pdaRun;
 }
 
-
 void Compiler::parsePatterns()
 {
-	Program *prg = colm_new_program( runtimeData );
+	program_t *prg = colm_new_program( runtimeData );
+
+	colm_set_debug( prg, gblActiveRealm );
 
 	/* Turn off context-dependent parsing. */
-	prg->ctxDepParsing = 0;
+	prg->ctx_dep_parsing = 0;
 
-	Tree **sp = prg->stackRoot;
+	tree_t **sp = prg->stack_root;
 
 	for ( ConsList::Iter cons = replList; cons.lte(); cons++ ) {
-		StreamImpl *in = newSourceStreamCons( "<internal>", cons );
-		cons->pdaRun = parsePattern( prg, sp, cons->loc, cons->langEl->parserId, in );
+		if ( cons->langEl != 0 ) {
+			struct stream_impl *in = colm_impl_new_cons( strdup("<internal>"), cons );
+			cons->pdaRun = parsePattern( prg, sp, cons->loc, cons->langEl->parserId, in );
+		}
 	}
 
 	for ( PatList::Iter pat = patternList; pat.lte(); pat++ ) {
-		StreamImpl *in = newSourceStreamPat( "<internal>", pat );
+		struct stream_impl *in = colm_impl_new_pat( strdup("<internal>"), pat );
 		pat->pdaRun = parsePattern( prg, sp, pat->loc, pat->langEl->parserId, in );
 	}
 
@@ -1038,8 +1073,7 @@ void Compiler::collectParserEls( BstSet<LangEl*> &parserEls )
 	}
 
 	for ( ConsList::Iter repl = replList; repl.lte(); repl++ ) {
-		/* We assume the reduction action compilation phase was run before
-		 * replacement parsing decorated the replacement with the target type. */
+		/* We need the the language element from the compilation process. */
 		assert( repl->langEl != 0 );
 
 		if ( repl->langEl->parserId < 0 ) {
@@ -1056,7 +1090,59 @@ void Compiler::collectParserEls( BstSet<LangEl*> &parserEls )
 	}
 }
 
-void Compiler::generateOutput( long activeRealm )
+void Compiler::writeHostCall()
+{
+	/*
+	 * Host Call
+	 */
+	for ( FunctionList::Iter hc = inHostList; hc.lte(); hc++ ) {
+		*outStream <<
+			"value_t " << hc->hostCall << "( program_t *prg, tree_t **sp";
+		for ( ParameterList::Iter p = *hc->paramList; p.lte(); p++ ) {
+			*outStream <<
+				", value_t";
+		}
+		*outStream << " );\n";
+	}
+
+	*outStream <<
+		"tree_t **" << objectName << "_host_call( program_t *prg, long code, tree_t **sp )\n"
+		"{\n"
+		"	value_t rtn = 0;\n"
+		"	switch ( code ) {\n";
+	
+	for ( FunctionList::Iter hc = inHostList; hc.lte(); hc++ ) {
+		*outStream <<
+			"		case " << hc->funcId << ": {\n";
+
+		int pos = hc->paramList->length() - 1;
+		for ( ParameterList::Iter p = *hc->paramList; p.lte(); p++, pos-- ) {
+			*outStream <<
+				"			value_t p" << pos << " = vm_pop_value();\n";
+		}
+		
+		*outStream <<
+			"			rtn = " << hc->hostCall << "( prg, sp";
+
+		pos = 0;
+		for ( ParameterList::Iter p = *hc->paramList; p.lte(); p++, pos++ ) {
+			*outStream <<
+				", p" << pos;
+		}
+		*outStream << " );\n"
+			"			break;\n"
+			"		}\n";
+	}
+		
+	*outStream <<
+		"	}\n"
+		"	vm_push_value( rtn );\n"
+		"	return sp;\n"
+		"}\n";
+
+}
+
+void Compiler::generateOutput( long activeRealm, bool includeCommit )
 {
 	FsmCodeGen *fsmGen = new FsmCodeGen( *outStream, redFsm, fsmTables );
 
@@ -1072,6 +1158,11 @@ void Compiler::generateOutput( long activeRealm )
 	/* Write the runtime data. */
 	pdaGen->writeRuntimeData( runtimeData, pdaTables );
 
+	writeHostCall();
+
+	if ( includeCommit )
+		writeCommitStub();
+
 	if ( !gblLibrary ) 
 		fsmGen->writeMain( activeRealm );
 
@@ -1085,13 +1176,14 @@ void Compiler::prepGrammar()
 	wrapNonTerminals();
 
 	makeLangElIds();
+	makeStructElIds();
 	makeLangElNames();
 	makeDefinitionNames();
 	noUndefindLangEls();
 
 	/* Put the language elements in an index by language element id. */
-	langElIndex = new LangEl*[nextSymbolId+1];
-	memset( langElIndex, 0, sizeof(LangEl*)*(nextSymbolId+1) );
+	langElIndex = new LangEl*[nextLelId+1];
+	memset( langElIndex, 0, sizeof(LangEl*)*(nextLelId+1) );
 	for ( LelList::Iter lel = langEls; lel.lte(); lel++ )
 		langElIndex[lel->id] = lel;
 
@@ -1100,7 +1192,7 @@ void Compiler::prepGrammar()
 	/* Allocate the Runtime data now. Every PdaTable that we make 
 	 * will reference it, but it will be filled in after all the tables are
 	 * built. */
-	runtimeData = new RuntimeData;
+	runtimeData = new colm_sections;
 }
 
 void Compiler::compile()
@@ -1126,6 +1218,7 @@ void Compiler::compile()
 	prepGrammar();
 
 	placeAllLanguageObjects();
+	placeAllStructObjects();
 	placeAllFrameObjects();
 	placeAllFunctions();
 
