@@ -1,37 +1,59 @@
 /*
- *  Copyright 2001-2012 Adrian Thurston <thurston@complang.org>
+ * Copyright 2001-2012 Adrian Thurston <thurston@colm.net>
+ *
+ * Permission is hereby granted, free of charge, to any person obtaining a copy
+ * of this software and associated documentation files (the "Software"), to
+ * deal in the Software without restriction, including without limitation the
+ * rights to use, copy, modify, merge, publish, distribute, sublicense, and/or
+ * sell copies of the Software, and to permit persons to whom the Software is
+ * furnished to do so, subject to the following conditions:
+ *
+ * The above copyright notice and this permission notice shall be included in all
+ * copies or substantial portions of the Software.
+ *
+ * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
+ * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
+ * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
+ * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
+ * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
+ * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
+ * SOFTWARE.
  */
 
-#ifndef _PARSEDATA_H
-#define _PARSEDATA_H
+#ifndef _COLM_PARSEDATA_H
+#define _COLM_PARSEDATA_H
+
+#include <limits.h>
 
 #include <iostream>
-#include <limits.h>
-#include "bstset.h"
+
+#include <avlmap.h>
+#include <avlset.h>
+#include <bstmap.h>
+#include <vector.h>
+#include <bstset.h>
+#include <dlist.h>
+#include <dlistmel.h>
+#include <fsmgraph.h>
+#include <compare.h>
+
 #include "global.h"
-#include "avlmap.h"
-#include "avlset.h"
-#include "bstmap.h"
-#include "vector.h"
-#include "dlist.h"
-#include "dlistmel.h"
-#include "fsmgraph.h"
-#include "compare.h"
-#include "vector.h"
 #include "keyops.h"
 #include "parsetree.h"
 #include "cstring.h"
 #include "pdagraph.h"
-#include "compare.h"
 #include "pdarun.h"
 #include "bytecode.h"
 #include "program.h"
+#include "internal.h"
 
 using std::ostream;
 
 struct exit_object { };
 extern exit_object endp;
 void operator<<( std::ostream &out, exit_object & );
+extern const char *objectName;
+extern bool hostAdapters;
 
 /* Forwards. */
 struct RedFsm;
@@ -44,6 +66,8 @@ struct FsmCodeGen;
 #define REDUCE_CODE 0x2
 #define SHIFT_REDUCE_CODE 0x3
 
+typedef Vector<const char**> CharVectVect;
+
 /* This is used for tracking the current stack of include file/machine pairs. It is
  * is used to detect and recursive include structure. */
 struct IncludeStackItem
@@ -57,6 +81,18 @@ struct IncludeStackItem
 typedef Vector<IncludeStackItem> IncludeStack;
 typedef Vector<const char *> ArgsVector;
 
+struct DefineArg
+{
+	DefineArg( String name, String value )
+		: name(name), value(value) {}
+
+	String name;
+	String value;
+};
+
+typedef Vector<DefineArg> DefineVector;
+
+extern DefineVector defineArgs;
 extern ArgsVector includePaths;
 
 inline long makeReduceCode( long reduction, bool isShiftReduce )
@@ -70,11 +106,11 @@ struct ProdElList;
 struct PdaLiteral;
 struct Production;
 
-/* A pointer to this is in PdaRun, but it's specification is not known by the
+/* A pointer to this is in struct pda_run, but it's specification is not known by the
  * runtime code. The runtime functions that access it are defined in
  * ctinput.cpp and stubbed in fsmcodegen.cpp */
-struct Bindings
-	: public Vector<ParseTree*>
+struct bindings
+	: public Vector<parse_tree_t*>
 {};
 
 struct DefListEl { Production *prev, *next; };
@@ -232,7 +268,6 @@ struct LangEl : public DListEl<LangEl>
 
 	Type type;
 	long id;
-	bool isContext;
 	String displayString;
 	long numAppearances;
 	bool commit;
@@ -256,7 +291,7 @@ struct LangEl : public DListEl<LangEl>
 	LangEl *eofLel;
 
 	PdaGraph *pdaGraph;
-	PdaTables *pdaTables;
+	struct pda_tables *pdaTables;
 
 	PdaState *startState;
 
@@ -267,15 +302,13 @@ struct LangEl : public DListEl<LangEl>
 	long thisSize;
 	long ofiOffset;
 
-	GenericType *generic;
-
 	long parserId;
 
 	PredType predType;
 	long predValue;
 
-	Context *contextDef;
-	Context *contextIn;
+	StructDef *contextDef;
+	StructDef *contextIn;
 	bool noPreIgnore;
 	bool noPostIgnore;
 	bool isZero;
@@ -452,7 +485,7 @@ struct ReOrBlock;
 struct ReOrItem;
 struct TokenRegion;
 
-/* Tree of instantiated names. */
+/* tree_t of instantiated names. */
 typedef BstMapEl<String, NameInst*> NameMapEl;
 typedef BstMap<String, NameInst*, CmpStr> NameMap;
 typedef Vector<NameInst*> NameVect;
@@ -541,6 +574,7 @@ struct Compiler
 
 	void placeFrameFields( ObjectDef *localFrame );
 	void placeUserFunction( Function *func, bool isUserIter );
+	void placeAllStructObjects();
 	void placeAllLanguageObjects();
 	void placeAllFrameObjects();
 	void placeAllFunctions();
@@ -626,6 +660,8 @@ struct Compiler
 
 	UniqueType *mainReturnUT;
 
+	CharVectVect streamFileNames;
+
 	/* CONTEXT FREE */
 	ProdElList *makeProdElList( LangEl *langEl );
 	void wrapNonTerminals();
@@ -633,11 +669,15 @@ struct Compiler
 	void noUndefindLangEls();
 	void declareBaseLangEls();
 	void makeLangElIds();
+	void makeStructElIds();
 	void makeLangElNames();
 	void makeTerminalWrappers();
 	void makeEofElements();
 	void makeIgnoreCollectors();
 	void resolvePrecedence();
+	void resolveReductionActions();
+	void findReductionActionProds();
+	void resolveReducers();
 
 	void declarePass();
 	void resolvePass();
@@ -674,7 +714,7 @@ struct Compiler
 	bool makeFirstSetProd( Production *prod, PdaState *state );
 	void makeFirstSets();
 
-	int findIndexOff( PdaTables *pdaTables, PdaGraph *pdaGraph,
+	int findIndexOff( struct pda_tables *pdaTables, PdaGraph *pdaGraph,
 			PdaState *state, int &currLen );
 	void trySetTime( PdaTrans *trans, long code, long &time );
 	void addRegion( PdaState *tabState, PdaTrans *pdaTrans, long pdaKey,
@@ -707,7 +747,7 @@ struct Compiler
 	void addMatchText( ObjectDef *frame, LangEl *lel );
 	void addMatchLength( ObjectDef *frame, LangEl *lel );
 	void addInput( ObjectDef *frame );
-	void addCtx( ObjectDef *frame );
+	void addThis( ObjectDef *frame );
 	void addTransTokVar( ObjectDef *frame, LangEl *lel );
 	void addProdRHSVars( ObjectDef *localFrame, ProdElList *prodElList );
 	void addProdRedObjectVar( ObjectDef *localFrame, LangEl *langEl );
@@ -718,16 +758,16 @@ struct Compiler
 	void addPushBackLHS( Production *prod, CodeVect &code, long &insertPos );
 
 	void prepGrammar();
-	PdaRun *parsePattern( Program *prg, Tree **sp, const InputLoc &loc,
-			int parserId, StreamImpl *sourceStream );
+	struct pda_run *parsePattern( program_t *prg, tree_t **sp, const InputLoc &loc,
+			int parserId, struct stream_impl *sourceStream );
 	void parsePatterns();
 
 	void collectParserEls( LangElSet &parserEls );
 	void makeParser( LangElSet &parserEls );
 	PdaGraph *makePdaGraph( BstSet<LangEl*> &parserEls  );
-	PdaTables *makePdaTables( PdaGraph *pdaGraph );
+	struct pda_tables *makePdaTables( PdaGraph *pdaGraph );
 
-	void fillInPatterns( Program *prg );
+	void fillInPatterns( program_t *prg );
 	void makeRuntimeData();
 
 	/* Generate and write out the fsm. */
@@ -736,37 +776,42 @@ struct Compiler
 	void verifyParseStopGrammar( LangEl *langEl, PdaGraph *pdaGraph );
 	void computeAdvanceReductions( LangEl *langEl, PdaGraph *pdaGraph );
 
-	void initMapFunctions( GenericType *gen );
-	void initListField( GenericType *gen, const char *name, int offset );
+	void initListElField( GenericType *gen, const char *name, int offset );
+	void initListFieldEl( GenericType *gen, const char *name, int offset );
+	void initListFieldVal( GenericType *gen, const char *name, int offset );
+
 	void initListFields( GenericType *gen );
 	void initListFunctions( GenericType *gen );
+
+	void initMapElKey( GenericType *gen, const char *name, int offset );
+	void initMapElField( GenericType *gen, const char *name, int offset );
+	void initMapField( GenericType *gen, const char *name, int offset );
+
+	void initMapFields( GenericType *gen );
+	void initMapFunctions( GenericType *gen );
+
 	void initVectorFunctions( GenericType *gen );
-	void initParserField( GenericType *gen, const char *name, int offset, TypeRef *typeRef );
+	void initParserField( GenericType *gen, const char *name,
+			int offset, TypeRef *typeRef );
 	void initParserFunctions( GenericType *gen );
 	void initParserFields( GenericType *gen );
-	void initCtxField( GenericType *gen );
-
-	void initList2Field( GenericType *gen, const char *name, int offset );
-	void initList2ElFields( GenericType *gen );
-	void initList2ElField( GenericType *gen, const char *name, int offset );
-	void initList2Functions( GenericType *gen );
-	void initList2Fields( GenericType *gen );
-	void initMap2Functions( GenericType *gen );
 
 	void addStdin();
 	void addStdout();
 	void addStderr();
 	void addArgv();
 	void addError();
+	void addDefineArgs();
 	int argvOffset();
-	int argv0_Offset();
+	int arg0Offset();
 	void makeDefaultIterators();
-	void addLengthField( ObjectDef *objDef, Code getLength );
+	void addLengthField( ObjectDef *objDef, code_t getLength );
 	ObjectDef *findObject( const String &name );
 	void resolveListElementOf( ObjectDef *container, ObjectDef *obj, ElementOf *elof );
 	void resolveMapElementOf( ObjectDef *container, ObjectDef *obj, ElementOf *elof );
 	void resolveElementOf( ObjectDef *obj );
 	void makeFuncVisible( Function *func, bool isUserIter );
+	void makeInHostVisible( Function *func );
 
 	void declareFunction( Function *func );
 	void declareReductionCode( Production *prod );
@@ -776,12 +821,12 @@ struct Compiler
 	void declareByteCode();
 
 	void resolveFunction( Function *func );
+	void resolveInHost( Function *func );
 	void resolvePreEof( TokenRegion *region );
 	void resolveRootBlock();
 	void resolveTranslateBlock( LangEl *langEl );
 	void resolveReductionCode( Production *prod );
 	void resolveParseTree();
-	void resolveGenericTypes();
 
 	void compileFunction( Function *func, CodeVect &code );
 	void compileFunction( Function *func );
@@ -797,7 +842,7 @@ struct Compiler
 	void compileByteCode();
 
 	void resolveUses();
-	void generateOutput( long activeRealm );
+	void generateOutput( long activeRealm, bool includeCommit );
 	void compile();
 
 	void openNameSpace( ostream &out, Namespace *nspace );
@@ -806,7 +851,8 @@ struct Compiler
 	void generateExports();
 	void generateExportsImpl();
 
-	LocalInfo *makeLocalInfo( Locals &locals );
+	struct local_info *makeLocalInfo( Locals &locals );
+	short *makeTrees( ObjectDef *objectDef, int &numTrees );
 
 	/* 
 	 * Graphviz Generation
@@ -821,6 +867,7 @@ struct Compiler
 	 */
 
 	LelList langEls;
+	StructElList structEls;
 	DefList prodList;
 
 	/* Dumping. */
@@ -845,10 +892,7 @@ struct Compiler
 
 	LangEl *ptrLangEl;
 	LangEl *voidLangEl;
-	LangEl *boolLangEl;
-	LangEl *intLangEl;
 	LangEl *strLangEl;
-	LangEl *streamLangEl;
 	LangEl *anyLangEl;
 	LangEl *rootLangEl;
 	LangEl *noTokenLangEl;
@@ -858,7 +902,7 @@ struct Compiler
 
 	Namespace *rootNamespace;
 
-	int nextSymbolId;
+	int nextLelId;
 	int firstNonTermId;
 
 	LangEl **langElIndex;
@@ -872,15 +916,29 @@ struct Compiler
 	ConsList replList;
 	ParserTextList parserTextList;
 
+	StructDef *global;
+	StructEl *globalSel;
 	ObjectDef *globalObjectDef;
-	ObjectField *argv0;
-	ObjectField *argvList;
+	ObjectField *arg0;
+	ObjectField *argv;
+	StructDef *argvEl;
+	StructEl *argvElSel;
+
+	StructDef *stream;
+	StructEl *streamSel;
 
 	VectorTypeIdMap vectorTypeIdMap;
 
 	UniqueType *findUniqueType( enum TYPE typeId );
 	UniqueType *findUniqueType( enum TYPE typeId, LangEl *langEl );
 	UniqueType *findUniqueType( enum TYPE typeId, IterDef *iterDef );
+	UniqueType *findUniqueType( enum TYPE typeId, StructEl *structEl );
+	UniqueType *findUniqueType( enum TYPE typeId, GenericType *generic );
+
+	UniqueGeneric *findUniqueGeneric( UniqueGeneric::Type type,
+			UniqueType *utKey, UniqueType *utValue );
+	UniqueGeneric *findUniqueGeneric( UniqueGeneric::Type type,
+			UniqueType *utValue );
 
 	UniqueType *uniqueTypeNil;
 	UniqueType *uniqueTypeVoid;
@@ -888,25 +946,21 @@ struct Compiler
 	UniqueType *uniqueTypeBool;
 	UniqueType *uniqueTypeInt;
 	UniqueType *uniqueTypeStr;
-	UniqueType *uniqueTypeStream;
 	UniqueType *uniqueTypeIgnore;
 	UniqueType *uniqueTypeAny;
 
+	UniqueType *uniqueTypeStream;
+
 	UniqueTypeMap uniqeTypeMap;
 	UniqueRepeatMap uniqeRepeatMap;
-	UniqueMapMap uniqueMapMap;
-	UniqueListMap uniqueListMap;
-	UniqueVectorMap uniqueVectorMap;
-	UniqueParserMap uniqueParserMap;
-
-	UniqueList2ElMap  uniqueList2ElMap;
-	UniqueList2Map    uniqueList2Map;
-	UniqueMap2ElMap   uniqueMap2ElMap;
-	UniqueMap2Map     uniqueMap2Map;
+	UniqueGenericMap uniqueGenericMap;
 
 	void declareGlobalFields();
 	void declareStrFields();
+
+	void declareStreamField( ObjectDef *objDef, code_t getLength );
 	void declareStreamFields();
+
 	void declareIntFields();
 	void declareTokenFields();
 
@@ -914,14 +968,16 @@ struct Compiler
 	ObjectDef *strObj;
 	ObjectDef *streamObj;
 
-	FsmTables *fsmTables;
-	RuntimeData *runtimeData;
+	struct fsm_tables *fsmTables;
+	struct colm_sections *runtimeData;
 
 	int nextPatConsId;
 	int nextGenericId;
 
 	FunctionList functionList;
+	FunctionList inHostList;
 	int nextFuncId;
+	int nextHostId;
 
 	enum CompileContext {
 		CompileTranslation,
@@ -935,8 +991,8 @@ struct Compiler
 	LongVect breakJumps;
 	Function *curFunction;
 
-	/* Loops fill this in for return statements to use. */
-	CodeVect *loopCleanup;
+	/* For stack unwinding. Used at exits, returns, iterator destroy, etc. */
+	CodeVect unwindCode;
 
 	ObjectField *makeDataEl();
 	ObjectField *makePosEl();
@@ -965,7 +1021,7 @@ struct Compiler
 	RedFsm *redFsm;
 
 	PdaGraph *pdaGraph;
-	PdaTables *pdaTables;
+	struct pda_tables *pdaTables;
 
 	long predValue;
 	long nextMatchEndNum;
@@ -976,11 +1032,20 @@ struct Compiler
 	int contiguousOffset;
 	int contiguousStretch;
 
-	bool beginContiguous( CodeVect &code, int stretch );
-	void endContiguous( CodeVect &code, bool resetContiguous );
-	void clearContiguous( CodeVect &code, bool resetContiguous );
-
 	void declareReVars();
+
+	void initReductionNeeds( Reduction *reduction );
+	void loadRefs( Reduction *reduction, Production *production,
+			const ReduceTextItemList &list );
+
+	void writeHostCall();
+	void writeNeeds();
+	void writeCommit();
+	void writeLhsRef( Production *production, ReduceTextItem *i );
+	void writeRhsRef( Production *production, ReduceTextItem *i );
+	void writeRhsLoc( Production *production, ReduceTextItem *i );
+	void writeHostItemList( Production *production, const ReduceTextItemList &list );
+	void writeCommitStub();
 };
 
 void afterOpMinimize( FsmGraph *fsm, bool lastInSeq = true );
@@ -1004,23 +1069,44 @@ typedef AvlMapEl<String, ColmParser *> ParserDictEl;
 
 LangEl *declareLangEl( Compiler *pd, Namespace *nspace,
 		const String &data, LangEl::Type type );
-LangEl *declareStruct( Compiler *pd, Namespace *nspace,
-		const String &data, LangEl::Type type );
 LangEl *addLangEl( Compiler *pd, Namespace *nspace,
 		const String &data, LangEl::Type type );
 
+StructEl *declareStruct( Compiler *pd, Namespace *nspace,
+		const String &data, StructDef *context );
+
 void declareTypeAlias( Compiler *pd, Namespace *nspace,
 		const String &data, TypeRef *typeRef );
 
 LangEl *findType( Compiler *pd, Namespace *nspace, const String &data );
 
 ObjectMethod *initFunction( UniqueType *retType, ObjectDef *obj, 
-		const String &name, int methIdWV, int methIdWC, bool isConst );
+		const String &name, int methIdWV, int methIdWC,
+		bool isConst, bool useFnInstr = false, GenericType *useGeneric = 0 );
+
 ObjectMethod *initFunction( UniqueType *retType, ObjectDef *obj, 
 		const String &name, int methIdWV, int methIdWC,
-		UniqueType *arg1, bool isConst );
+		UniqueType *arg1, bool isConst, bool useFnInstr = false,
+		GenericType *useGeneric = 0 );
+
 ObjectMethod *initFunction( UniqueType *retType, ObjectDef *obj, 
 		const String &name, int methIdWV, int methIdWC, 
-		UniqueType *arg1, UniqueType *arg2, bool isConst );
+		UniqueType *arg1, UniqueType *arg2, bool isConst,
+		bool useFnInstr = false, GenericType *useGeneric = 0 );
+
+ObjectMethod *initFunction( UniqueType *retType, Namespace *nspace, ObjectDef *obj, 
+		const String &name, int methIdWV, int methIdWC,
+		bool isConst, bool useFnInstr = false, GenericType *useGeneric = 0 );
+
+ObjectMethod *initFunction( UniqueType *retType, Namespace *nspace, ObjectDef *obj, 
+		const String &name, int methIdWV, int methIdWC,
+		UniqueType *arg1, bool isConst, bool useFnInstr = false,
+		GenericType *useGeneric = 0 );
+
+ObjectMethod *initFunction( UniqueType *retType, Namespace *nspace, ObjectDef *obj, 
+		const String &name, int methIdWV, int methIdWC, 
+		UniqueType *arg1, UniqueType *arg2, bool isConst,
+		bool useFnInstr = false, GenericType *useGeneric = 0 );
+
+#endif /* _COLM_PARSEDATA_H */
 
-#endif /* _PARSEDATA_H */
