@@ -1,19 +1,29 @@
 /*
- *  Copyright 2006-2012 Adrian Thurston <thurston@complang.org>
+ * Copyright 2006-2015 Adrian Thurston <thurston@colm.net>
+ *
+ * Permission is hereby granted, free of charge, to any person obtaining a copy
+ * of this software and associated documentation files (the "Software"), to
+ * deal in the Software without restriction, including without limitation the
+ * rights to use, copy, modify, merge, publish, distribute, sublicense, and/or
+ * sell copies of the Software, and to permit persons to whom the Software is
+ * furnished to do so, subject to the following conditions:
+ *
+ * The above copyright notice and this permission notice shall be included in all
+ * copies or substantial portions of the Software.
+ *
+ * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
+ * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
+ * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
+ * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
+ * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
+ * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
+ * SOFTWARE.
  */
 
-#include <iostream>
-#include <errno.h>
-
-#include "parser.h"
-#include "config.h"
-#include "avltree.h"
-#include "compiler.h"
-#include "parser.h"
-#include "global.h"
-#include "input.h"
 #include "consinit.h"
 
+#include <iostream>
+
 using std::cout;
 using std::cerr;
 using std::endl;
@@ -437,20 +447,23 @@ void ConsInit::definition( const String &name, Production *prod1, Production *pr
 	prodAppend( defList, prod3 );
 	prodAppend( defList, prod4 );
 
-	NtDef *ntDef = NtDef::cons( name, curNspace(), curContext(), false );
-	ObjectDef *objectDef = ObjectDef::cons( ObjectDef::UserType, name, pd->nextObjectId++ ); 
+	NtDef *ntDef = NtDef::cons( name, curNspace(), curStruct(), false );
+	ObjectDef *objectDef = ObjectDef::cons( ObjectDef::UserType,
+			name, pd->nextObjectId++ ); 
 	cflDef( ntDef, objectDef, defList );
 }
 
-void ConsInit::definition( const String &name, Production *prod1, Production *prod2, Production *prod3 )
+void ConsInit::definition( const String &name, Production *prod1,
+		Production *prod2, Production *prod3 )
 {
 	LelDefList *defList = new LelDefList;
 	prodAppend( defList, prod1 );
 	prodAppend( defList, prod2 );
 	prodAppend( defList, prod3 );
 
-	NtDef *ntDef = NtDef::cons( name, curNspace(), curContext(), false );
-	ObjectDef *objectDef = ObjectDef::cons( ObjectDef::UserType, name, pd->nextObjectId++ ); 
+	NtDef *ntDef = NtDef::cons( name, curNspace(), curStruct(), false );
+	ObjectDef *objectDef = ObjectDef::cons( ObjectDef::UserType,
+			name, pd->nextObjectId++ ); 
 	cflDef( ntDef, objectDef, defList );
 }
 
@@ -460,8 +473,9 @@ void ConsInit::definition( const String &name, Production *prod1, Production *pr
 	prodAppend( defList, prod1 );
 	prodAppend( defList, prod2 );
 
-	NtDef *ntDef = NtDef::cons( name, curNspace(), curContext(), false );
-	ObjectDef *objectDef = ObjectDef::cons( ObjectDef::UserType, name, pd->nextObjectId++ ); 
+	NtDef *ntDef = NtDef::cons( name, curNspace(), curStruct(), false );
+	ObjectDef *objectDef = ObjectDef::cons( ObjectDef::UserType,
+			name, pd->nextObjectId++ ); 
 	cflDef( ntDef, objectDef, defList );
 }
 
@@ -470,8 +484,9 @@ void ConsInit::definition( const String &name, Production *prod )
 	LelDefList *defList = new LelDefList;
 	prodAppend( defList, prod );
 
-	NtDef *ntDef = NtDef::cons( name, curNspace(), curContext(), false );
-	ObjectDef *objectDef = ObjectDef::cons( ObjectDef::UserType, name, pd->nextObjectId++ ); 
+	NtDef *ntDef = NtDef::cons( name, curNspace(), curStruct(), false );
+	ObjectDef *objectDef = ObjectDef::cons( ObjectDef::UserType,
+			name, pd->nextObjectId++ ); 
 	cflDef( ntDef, objectDef, defList );
 }
 
@@ -527,14 +542,19 @@ void ConsInit::lexFactorRep()
 void ConsInit::lexTerm()
 {
 	ProdEl *prodEl1 = prodRefName( "Term", "lex_term" );
-	ProdEl *prodEl2 = prodRefLit( "'.'" );
+	ProdEl *prodEl2 = prodRefName( "Dot", "DOT" );
 	ProdEl *prodEl3 = prodRefName( "FactorRep", "lex_factor_rep" );
 	Production *prod1 = production( prodEl1, prodEl2, prodEl3 );
-	
-	ProdEl *prodEl4 = prodRefName( "FactorRep", "lex_factor_rep" );
-	Production *prod2 = production( prodEl4 );
 
-	definition( "lex_term", prod1, prod2 );
+	ProdEl *prodEl4 = prodRefName( "Term", "lex_term" );
+	ProdEl *prodEl5 = prodRefName( "ColonLt", "COLON_LT" );
+	ProdEl *prodEl6 = prodRefName( "FactorRep", "lex_factor_rep" );
+	Production *prod2 = production( prodEl4, prodEl5, prodEl6 );
+	
+	ProdEl *prodEl7 = prodRefName( "FactorRep", "lex_factor_rep" );
+	Production *prod3 = production( prodEl7 );
+
+	definition( "lex_term", prod1, prod2, prod3 );
 }
 
 void ConsInit::lexExpr()
@@ -722,60 +742,82 @@ void ConsInit::startProd()
 
 void ConsInit::parseInput( StmtList *stmtList )
 {
-	/* Parse the "start" def. */
-	NamespaceQual *nspaceQual = NamespaceQual::cons( curNspace() );
-	TypeRef *typeRef = TypeRef::cons( internal, nspaceQual, String("start"), RepeatNone );
-
 	/* Pop argv, this yields the file name . */
 	CallArgVect *popArgs = new CallArgVect;
 	QualItemVect *popQual = new QualItemVect;
 	popQual->append( QualItem( QualItem::Arrow, internal, String( "argv" ) ) );
 
 	LangVarRef *popRef = LangVarRef::cons( internal,
-			0, curLocalFrame->rootScope, popQual, String("pop") );
+			curNspace(), 0, curLocalFrame()->rootScope,
+			NamespaceQual::cons( curNspace() ), popQual, String("pop") );
 	LangExpr *pop = LangExpr::cons( LangTerm::cons( InputLoc(), popRef, popArgs ) );
 
+	TypeRef *typeRef = TypeRef::cons( internal, pd->uniqueTypeStr );
+	ObjectField *objField = ObjectField::cons( internal,
+			ObjectField::UserLocalType, typeRef, "A" );
+
+	LangStmt *stmt = varDef( objField, pop, LangStmt::AssignType );
+	stmtList->append( stmt );
+
 	/* Construct a literal string 'r', for second arg to open. */
-	ConsItem *modeConsItem = ConsItem::cons( internal, ConsItem::InputText, String("r") );
+	ConsItem *modeConsItem = ConsItem::cons( internal,
+			ConsItem::InputText, String("r") );
 	ConsItemList *modeCons = new ConsItemList;
 	modeCons->append( modeConsItem );
 	LangExpr *modeExpr = LangExpr::cons( LangTerm::cons( internal, modeCons ) );
+
+	/* Reference A->value */
+	QualItemVect *qual = new QualItemVect;
+	LangVarRef *varRef = LangVarRef::cons( internal, curNspace(), 0,
+			curLocalFrame()->rootScope, NamespaceQual::cons( curNspace() ),
+			qual, String("A") );
+	LangExpr *Avalue = LangExpr::cons( LangTerm::cons( internal,
+			LangTerm::VarRefType, varRef ) );
 	
 	/* Call open. */
 	QualItemVect *openQual = new QualItemVect;
 	LangVarRef *openRef = LangVarRef::cons( internal,
-			0, curLocalFrame->rootScope, openQual, String("open") );
+			0, 0, curLocalFrame()->rootScope,
+			NamespaceQual::cons( curNspace() ), openQual, String("open") );
 	CallArgVect *openArgs = new CallArgVect;
-	openArgs->append( new CallArg(pop) );
+	openArgs->append( new CallArg(Avalue) );
 	openArgs->append( new CallArg(modeExpr) );
 	LangExpr *open = LangExpr::cons( LangTerm::cons( InputLoc(), openRef, openArgs ) );
 
 	/* Construct a list containing the open stream. */
-	ConsItem *consItem = ConsItem::cons( internal, ConsItem::ExprType, open );
+	ConsItem *consItem = ConsItem::cons( internal, ConsItem::ExprType, open, false );
 	ConsItemList *list = ConsItemList::cons( consItem );
 
 	/* Will capture the parser to "P" */
-	ObjectField *objField = ObjectField::cons( internal,
+	objField = ObjectField::cons( internal,
 			ObjectField::UserLocalType, 0, String("P") );
 
+	/* Parse the "start" def. */
+	NamespaceQual *nspaceQual = NamespaceQual::cons( curNspace() );
+	typeRef = TypeRef::cons( internal, nspaceQual,
+			String("start"), RepeatNone );
+
 	/* Parse the above list. */
-	LangExpr *parseExpr = parseCmd( internal, false, false, objField, typeRef, 0, list );
+	LangExpr *parseExpr = parseCmd( internal, false, false, objField,
+			typeRef, 0, list, true, false, "" );
 	LangStmt *parseStmt = LangStmt::cons( internal, LangStmt::ExprType, parseExpr );
 	stmtList->append( parseStmt );
 }
 
 void ConsInit::exportTree( StmtList *stmtList )
 {
+	/* reference P */
 	QualItemVect *qual = new QualItemVect;
-	LangVarRef *varRef = LangVarRef::cons( internal, 0,
-			curLocalFrame->rootScope, qual, String("P") );
+	LangVarRef *varRef = LangVarRef::cons( internal, curNspace(), 0,
+			curLocalFrame()->rootScope, NamespaceQual::cons( curNspace() ), qual, String("P") );
 	LangExpr *expr = LangExpr::cons( LangTerm::cons( internal,
 			LangTerm::VarRefType, varRef ) );
 
+	/* Assign P to ColmTree */
 	NamespaceQual *nspaceQual = NamespaceQual::cons( curNspace() );
 	TypeRef *typeRef = TypeRef::cons( internal, nspaceQual, String("start"), RepeatNone );
 	ObjectField *program = ObjectField::cons( internal,
-			ObjectField::UserFieldType, typeRef, String("ColmTree") );
+			ObjectField::StructFieldType, typeRef, String("ColmTree") );
 	LangStmt *programExport = exportStmt( program, LangStmt::AssignType, expr );
 	stmtList->append( programExport );
 }
@@ -810,7 +852,8 @@ void ConsInit::go( long activeRealm )
 	keyword( "'|'" );
 	keyword( "'/'" );
 	keyword( "':'" );
-	keyword( "'.'" );
+	keyword( "DOT", "'.'" );
+	keyword( "COLON_LT", "':>'" );
 	keyword( "'('" );
 	keyword( "')'" );
 	keyword( "'..'" );
