@@ -1,15 +1,33 @@
 /*
- *  Copyright 2006-2012 Adrian Thurston <thurston@complang.org>
+ * Copyright 2006-2012 Adrian Thurston <thurston@colm.net>
+ *
+ * Permission is hereby granted, free of charge, to any person obtaining a copy
+ * of this software and associated documentation files (the "Software"), to
+ * deal in the Software without restriction, including without limitation the
+ * rights to use, copy, modify, merge, publish, distribute, sublicense, and/or
+ * sell copies of the Software, and to permit persons to whom the Software is
+ * furnished to do so, subject to the following conditions:
+ *
+ * The above copyright notice and this permission notice shall be included in all
+ * copies or substantial portions of the Software.
+ *
+ * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
+ * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
+ * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
+ * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
+ * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
+ * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
+ * SOFTWARE.
  */
 
-#include "compiler.h"
-#include "fsmcodegen.h"
-#include "redfsm.h"
-#include "bstmap.h"
-#include <sstream>
-#include <string>
 #include <assert.h>
+#include <string.h>
+#include <stdbool.h>
 
+#include <sstream>
+#include <iostream>
+
+#include "fsmcodegen.h"
 
 using std::ostream;
 using std::ostringstream;
@@ -17,10 +35,9 @@ using std::string;
 using std::cerr;
 using std::endl;
 
-
 /* Init code gen with in parameters. */
 FsmCodeGen::FsmCodeGen( ostream &out, 
-		RedFsm *redFsm, FsmTables *fsmTables )
+		RedFsm *redFsm, fsm_tables *fsmTables )
 :
 	out(out),
 	redFsm(redFsm), 
@@ -78,7 +95,7 @@ string FsmCodeGen::CS()
 {
 	ostringstream ret;
 	/* Expression for retrieving the key, use simple dereference. */
-	ret << ACCESS() << "cs";
+	ret << ACCESS() << "fsm_cs";
 	return ret.str();
 }
 
@@ -760,10 +777,10 @@ void FsmCodeGen::writeData()
 	out << "\n";
 
 	out << "static long " << ENTRY_BY_REGION() << "[] = {\n\t";
-	for ( int i = 0; i < fsmTables->numRegions; i++ ) {
-		out << fsmTables->entryByRegion[i];
+	for ( int i = 0; i < fsmTables->num_regions; i++ ) {
+		out << fsmTables->entry_by_region[i];
 
-		if ( i < fsmTables->numRegions-1 ) {
+		if ( i < fsmTables->num_regions-1 ) {
 			out << ", ";
 			if ( (i+1) % 8 == 0 )
 				out << "\n\t";
@@ -772,7 +789,7 @@ void FsmCodeGen::writeData()
 	out << "\n};\n\n";
 
 	out <<
-		"static FsmTables fsmTables_start =\n"
+		"static struct fsm_tables fsmTables_start =\n"
 		"{\n"
 		"	0, "       /* actions */
 		" 0, "         /* keyOffsets */
@@ -831,9 +848,9 @@ void FsmCodeGen::writeExec()
 	setLabelsNeeded();
 
 	out <<
-		"static void fsmExecute( FsmRun *fsmRun, StreamImpl *inputStream )\n"
+		"static void fsm_execute( struct pda_run *pdaRun, struct stream_impl *inputStream )\n"
 		"{\n"
-		"	" << BLOCK_START() << " = fsmRun->p;\n"
+		"	" << BLOCK_START() << " = pdaRun->p;\n"
 		"/*_resume:*/\n";
 
 	if ( redFsm->errState != 0 ) {
@@ -873,26 +890,6 @@ void FsmCodeGen::writeExec()
 		"\n";
 }
 
-void FsmCodeGen::writeIncludes()
-{
-	out << 
-		"#include <colm/pdarun.h>\n"
-		"#include <colm/debug.h>\n"
-		"#include <colm/bytecode.h>\n"
-		"#include <stdio.h>\n"
-		"#include <stdlib.h>\n"
-		"#include <string.h>\n"
-		"#include <assert.h>\n"
-		"#include <colm/config.h>\n"
-		"#include <colm/defs.h>\n"
-		"#include <colm/input.h>\n"
-		"#include <colm/tree.h>\n"
-		"#include <colm/program.h>\n"
-		"#include <colm/colm.h>\n"
-		"\n"
-		"\n";
-}
-
 void FsmCodeGen::writeCode()
 {
 	redFsm->depthFirstOrdering();
@@ -903,10 +900,10 @@ void FsmCodeGen::writeCode()
 	/* Referenced in the runtime lib, but used only in the compiler. Probably
 	 * should use the preprocessor to make these go away. */
 	out <<
-		"static void sendNamedLangEl( Program *prg, Tree **tree, PdaRun *pdaRun,\n"
-		"		FsmRun *fsmRun, StreamImpl *inputStream ) { }\n"
-		"static void initBindings( PdaRun *pdaRun ) {}\n"
-		"static void popBinding( PdaRun *pdaRun, ParseTree *tree ) {}\n"
+		"static void sendNamedLangEl( struct colm_program *prg, tree_t **tree,\n"
+		"		struct pda_run *pdaRun, struct stream_impl *inputStream ) { }\n"
+		"static void initBindings( struct pda_run *pdaRun ) {}\n"
+		"static void popBinding( struct pda_run *pdaRun, parse_tree_t *tree ) {}\n"
 		"\n"
 		"\n";
 }
