@@ -1,22 +1,84 @@
 /*
- *  Copyright 2010-2012 Adrian Thurston <thurston@complang.org>
+ * Copyright 2010-2012 Adrian Thurston <thurston@colm.net>
+ *
+ * Permission is hereby granted, free of charge, to any person obtaining a copy
+ * of this software and associated documentation files (the "Software"), to
+ * deal in the Software without restriction, including without limitation the
+ * rights to use, copy, modify, merge, publish, distribute, sublicense, and/or
+ * sell copies of the Software, and to permit persons to whom the Software is
+ * furnished to do so, subject to the following conditions:
+ *
+ * The above copyright notice and this permission notice shall be included in all
+ * copies or substantial portions of the Software.
+ *
+ * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
+ * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
+ * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
+ * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
+ * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
+ * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
+ * SOFTWARE.
  */
 
+#include <colm/map.h>
 
 #include <assert.h>
+#include <stdbool.h>
+
 #include <colm/pdarun.h>
-#include <colm/map.h>
 #include <colm/pool.h>
+#include <colm/bytecode.h>
 
-#define true 1
-#define false 0
+struct colm_struct *colm_map_el_get( struct colm_program *prg,
+		map_el_t *map_el, word_t gen_id, word_t field )
+{
+	struct generic_info *gi = &prg->rtd->generic_info[gen_id];
+	map_el_t *result = 0;
+	switch ( field ) {
+		case 0: 
+			result = map_el->prev;
+			break;
+		case 1: 
+			result = map_el->next;
+			break;
+		default:
+			assert( 0 );
+			break;
+	}
 
-void mapListAbandon( Map *map )
+	struct colm_struct *s = result != 0 ?
+			colm_struct_container( result, gi->el_offset ) : 0;
+	return s;
+}
+
+struct colm_struct *colm_map_get( struct colm_program *prg,
+		map_t *map, word_t gen_id, word_t field )
+{
+	struct generic_info *gi = &prg->rtd->generic_info[gen_id];
+	map_el_t *result = 0;
+	switch ( field ) {
+		case 0: 
+			result = map->head;
+			break;
+		case 1: 
+			result = map->tail;
+			break;
+		default:
+			assert( 0 );
+			break;
+	}
+
+	struct colm_struct *s = result != 0 ?
+			colm_struct_container( result, gi->el_offset ) : 0;
+	return s;
+}
+
+void map_list_abandon( map_t *map )
 {
 	map->head = map->tail = 0;
 }
 
-void mapListAddBefore( Map *map, MapEl *next_el, MapEl *new_el )
+void map_list_add_before( map_t *map, map_el_t *next_el, map_el_t *new_el )
 {
 	/* Set the next pointer of the new element to next_el. We do
 	 * this regardless of the state of the list. */
@@ -45,7 +107,7 @@ void mapListAddBefore( Map *map, MapEl *next_el, MapEl *new_el )
 	}
 }
 
-void mapListAddAfter( Map *map, MapEl *prev_el, MapEl *new_el )
+void map_list_add_after( map_t *map, map_el_t *prev_el, map_el_t *new_el )
 {
 	/* Set the previous pointer of new_el to prev_el. We do
 	 * this regardless of the state of the list. */
@@ -75,7 +137,7 @@ void mapListAddAfter( Map *map, MapEl *prev_el, MapEl *new_el )
 }
 
 
-MapEl *mapListDetach( Map *map, MapEl *el )
+map_el_t *map_list_detach( map_t *map, map_el_t *el )
 {
 	/* Set forward pointers to skip over el. */
 	if ( el->prev == 0 ) 
@@ -95,13 +157,13 @@ MapEl *mapListDetach( Map *map, MapEl *el )
 
 
 /* Once an insertion position is found, attach a element to the tree. */
-void mapAttachRebal( Map *map, MapEl *element, MapEl *parentEl, MapEl *lastLess )
+void map_attach_rebal( map_t *map, map_el_t *element, map_el_t *parent_el, map_el_t *last_less )
 {
 	/* Increment the number of element in the tree. */
-	map->treeSize += 1;
+	map->tree_size += 1;
 
 	/* Set element's parent. */
-	element->parent = parentEl;
+	element->parent = parent_el;
 
 	/* New element always starts as a leaf with height 1. */
 	element->left = 0;
@@ -109,50 +171,50 @@ void mapAttachRebal( Map *map, MapEl *element, MapEl *parentEl, MapEl *lastLess
 	element->height = 1;
 
 	/* Are we inserting in the tree somewhere? */
-	if ( parentEl != 0 ) {
+	if ( parent_el != 0 ) {
 		/* We have a parent so we are somewhere in the tree. If the parent
 		 * equals lastLess, then the last traversal in the insertion went
 		 * left, otherwise it went right. */
-		if ( lastLess == parentEl ) {
-			parentEl->left = element;
+		if ( last_less == parent_el ) {
+			parent_el->left = element;
 
-			mapListAddBefore( map, parentEl, element );
+			map_list_add_before( map, parent_el, element );
 		}
 		else {
-			parentEl->right = element;
+			parent_el->right = element;
 
-			mapListAddAfter( map, parentEl, element );
+			map_list_add_after( map, parent_el, element );
 		}
 	}
 	else {
 		/* No parent element so we are inserting the root. */
 		map->root = element;
 
-		mapListAddAfter( map, map->tail, element );
+		map_list_add_after( map, map->tail, element );
 	}
 
 	/* Recalculate the heights. */
-	mapRecalcHeights( map, parentEl );
+	map_recalc_heights( map, parent_el );
 
 	/* Find the first unbalance. */
-	MapEl *ub = mapFindFirstUnbalGP( map, element );
+	map_el_t *ub = mapFindFirstUnbalGP( map, element );
 
 	/* rebalance. */
 	if ( ub != 0 )
 	{
 		/* We assert that after this single rotation the 
 		 * tree is now properly balanced. */
-		mapRebalance( map, ub );
+		map_rebalance( map, ub );
 	}
 }
 
 #if 0
 /* Recursively delete all the children of a element. */
-void mapDeleteChildrenOf( Map *map, MapEl *element )
+void map_delete_children_of( map_t *map, map_el_t *element )
 {
 	/* Recurse left. */
 	if ( element->left ) {
-		mapDeleteChildrenOf( map, element->left );
+		map_delete_children_of( map, element->left );
 
 		/* Delete left element. */
 		delete element->left;
@@ -161,7 +223,7 @@ void mapDeleteChildrenOf( Map *map, MapEl *element )
 
 	/* Recurse right. */
 	if ( element->right ) {
-		mapDeleteChildrenOf( map, element->right );
+		map_delete_children_of( map, element->right );
 
 		/* Delete right element. */
 		delete element->right;
@@ -169,31 +231,31 @@ void mapDeleteChildrenOf( Map *map, MapEl *element )
 	}
 }
 
-void mapEmpty( Map *map )
+void map_empty( map_t *map )
 {
 	if ( map->root ) {
 		/* Recursively delete from the tree structure. */
-		mapDeleteChildrenOf( map, map->root );
+		map_delete_children_of( map, map->root );
 		delete map->root;
 		map->root = 0;
-		map->treeSize = 0;
+		map->tree_size = 0;
 
-		mapListAbandon( map );
+		map_list_abandon( map );
 	}
 }
 #endif
 
 /* rebalance from a element whose gradparent is unbalanced. Only
  * call on a element that has a grandparent. */
-MapEl *mapRebalance( Map *map, MapEl *n )
+map_el_t *map_rebalance( map_t *map, map_el_t *n )
 {
 	long lheight, rheight;
-	MapEl *a, *b, *c;
-	MapEl *t1, *t2, *t3, *t4;
+	map_el_t *a, *b, *c;
+	map_el_t *t1, *t2, *t3, *t4;
 
-	MapEl *p = n->parent; /* parent (Non-NUL). L*/
-	MapEl *gp = p->parent; /* Grand-parent (Non-NULL). */
-	MapEl *ggp = gp->parent; /* Great grand-parent (may be NULL). */
+	map_el_t *p = n->parent; /* parent (Non-NUL). L*/
+	map_el_t *gp = p->parent; /* Grand-parent (Non-NULL). */
+	map_el_t *ggp = gp->parent; /* Great grand-parent (may be NULL). */
 
 	if (gp->right == p)
 	{
@@ -334,12 +396,12 @@ MapEl *mapRebalance( Map *map, MapEl *n )
 	b->height = (lheight > rheight ? lheight : rheight) + 1;
 
 	/* Fix height of b's parents. */
-	mapRecalcHeights( map, ggp );
+	map_recalc_heights( map, ggp );
 	return ggp;
 }
 
 /* Recalculates the heights of all the ancestors of element. */
-void mapRecalcHeights( Map *map, MapEl *element )
+void map_recalc_heights( map_t *map, map_el_t *element )
 {
 	while ( element != 0 )
 	{
@@ -361,10 +423,10 @@ void mapRecalcHeights( Map *map, MapEl *element )
 }
 
 /* Finds the first element whose grandparent is unbalanced. */
-MapEl *mapFindFirstUnbalGP( Map *map, MapEl *element )
+map_el_t *mapFindFirstUnbalGP( map_t *map, map_el_t *element )
 {
-	long lheight, rheight, balanceProp;
-	MapEl *gp;
+	long lheight, rheight, balance_prop;
+	map_el_t *gp;
 
 	if ( element == 0 || element->parent == 0 ||
 			element->parent->parent == 0 )
@@ -376,9 +438,9 @@ MapEl *mapFindFirstUnbalGP( Map *map, MapEl *element )
 	{
 		lheight = gp->left ? gp->left->height : 0;
 		rheight = gp->right ? gp->right->height : 0;
-		balanceProp = lheight - rheight;
+		balance_prop = lheight - rheight;
 
-		if ( balanceProp < -1 || balanceProp > 1 )
+		if ( balance_prop < -1 || balance_prop > 1 )
 			return element;
 
 		element = element->parent;
@@ -390,7 +452,7 @@ MapEl *mapFindFirstUnbalGP( Map *map, MapEl *element )
 
 
 /* Finds the first element that is unbalanced. */
-MapEl *mapFindFirstUnbalEl( Map *map, MapEl *element )
+map_el_t *map_find_first_unbal_el( map_t *map, map_el_t *element )
 {
 	if ( element == 0 )
 		return 0;
@@ -401,9 +463,9 @@ MapEl *mapFindFirstUnbalEl( Map *map, MapEl *element )
 			element->left->height : 0;
 		long rheight = element->right ?
 			element->right->height : 0;
-		long balanceProp = lheight - rheight;
+		long balance_prop = lheight - rheight;
 
-		if ( balanceProp < -1 || balanceProp > 1 )
+		if ( balance_prop < -1 || balance_prop > 1 )
 			return element;
 
 		element = element->parent;
@@ -412,9 +474,9 @@ MapEl *mapFindFirstUnbalEl( Map *map, MapEl *element )
 }
 
 /* Replace a element in the tree with another element not in the tree. */
-void mapReplaceEl( Map *map, MapEl *element, MapEl *replacement )
+void map_replace_el( map_t *map, map_el_t *element, map_el_t *replacement )
 {
-	MapEl *parent = element->parent,
+	map_el_t *parent = element->parent,
 			*left = element->left,
 			*right = element->right;
 
@@ -443,9 +505,9 @@ void mapReplaceEl( Map *map, MapEl *element, MapEl *replacement )
 
 /* Removes a element from a tree and puts filler in it's place.
  * Filler should be null or a child of element. */
-void mapRemoveEl( Map *map, MapEl *element, MapEl *filler )
+void map_remove_el( map_t *map, map_el_t *element, map_el_t *filler )
 {
-	MapEl *parent = element->parent;
+	map_el_t *parent = element->parent;
 
 	if ( parent )
 	{
@@ -464,138 +526,205 @@ void mapRemoveEl( Map *map, MapEl *element, MapEl *filler )
 	return;
 }
 
+#if 0
 /* Recursive worker for tree copying. */
-MapEl *mapCopyBranch( Program *prg, Map *map, MapEl *el, Kid *oldNextDown, Kid **newNextDown )
+map_el_t *map_copy_branch( program_t *prg, map_t *map, map_el_t *el, kid_t *old_next_down, kid_t **new_next_down )
 {
 	/* Duplicate element. Either the base element's copy constructor or defaul
 	 * constructor will get called. Both will suffice for initting the
 	 * pointers to null when they need to be. */
-	MapEl *newEl = mapElAllocate( prg );
+	map_el_t *new_el = map_el_allocate( prg );
 
-	if ( (Kid*)el == oldNextDown )
-		*newNextDown = (Kid*)newEl;
+	if ( (kid_t*)el == old_next_down )
+		*new_next_down = (kid_t*)new_el;
 
 	/* If the left tree is there, copy it. */
-	if ( newEl->left ) {
-		newEl->left = mapCopyBranch( prg, map, newEl->left, oldNextDown, newNextDown );
-		newEl->left->parent = newEl;
+	if ( new_el->left ) {
+		new_el->left = map_copy_branch( prg, map, new_el->left, old_next_down, new_next_down );
+		new_el->left->parent = new_el;
 	}
 
-	mapListAddAfter( map, map->tail, newEl );
+	map_list_add_after( map, map->tail, new_el );
 
 	/* If the right tree is there, copy it. */
-	if ( newEl->right ) {
-		newEl->right = mapCopyBranch( prg, map, newEl->right, oldNextDown, newNextDown );
-		newEl->right->parent = newEl;
+	if ( new_el->right ) {
+		new_el->right = map_copy_branch( prg, map, new_el->right, old_next_down, new_next_down );
+		new_el->right->parent = new_el;
 	}
 
-	return newEl;
+	return new_el;
+}
+#endif
+
+static long map_cmp( program_t *prg, map_t *map, const tree_t *tree1, const tree_t *tree2 )
+{
+	if ( map->generic_info->key_type == TYPE_TREE ) {
+		return colm_cmp_tree( prg, tree1, tree2 );
+	}
+	else {
+		if ( (long)tree1 < (long)tree2 )
+			return -1;
+		else if ( (long)tree1 > (long)tree2)
+			return 1;
+		return 0;
+	}
 }
 
-MapEl *mapInsertEl( Program *prg, Map *map, MapEl *element, MapEl **lastFound )
+map_el_t *map_insert_el( program_t *prg, map_t *map, map_el_t *element, map_el_t **last_found )
 {
-	long keyRelation;
-	MapEl *curEl = map->root, *parentEl = 0;
-	MapEl *lastLess = 0;
+	long key_relation;
+	map_el_t *cur_el = map->root, *parent_el = 0;
+	map_el_t *last_less = 0;
 
 	while ( true ) {
-		if ( curEl == 0 ) {
+		if ( cur_el == 0 ) {
 			/* We are at an external element and did not find the key we were
 			 * looking for. Attach underneath the leaf and rebalance. */
-			mapAttachRebal( map, element, parentEl, lastLess );
+			map_attach_rebal( map, element, parent_el, last_less );
 
-			if ( lastFound != 0 )
-				*lastFound = element;
+			if ( last_found != 0 )
+				*last_found = element;
 			return element;
 		}
 
-		keyRelation = cmpTree( prg,
-			element->key, curEl->key );
+		key_relation = map_cmp( prg, map,
+			element->key, cur_el->key );
 
 		/* Do we go left? */
-		if ( keyRelation < 0 ) {
-			parentEl = lastLess = curEl;
-			curEl = curEl->left;
+		if ( key_relation < 0 ) {
+			parent_el = last_less = cur_el;
+			cur_el = cur_el->left;
 		}
 		/* Do we go right? */
-		else if ( keyRelation > 0 ) {
-			parentEl = curEl;
-			curEl = curEl->right;
+		else if ( key_relation > 0 ) {
+			parent_el = cur_el;
+			cur_el = cur_el->right;
 		}
 		/* We have hit the target. */
 		else {
-			if ( lastFound != 0 )
-				*lastFound = curEl;
+			if ( last_found != 0 )
+				*last_found = cur_el;
 			return 0;
 		}
 	}
 }
 
-MapEl *mapInsertKey( Program *prg, Map *map, Tree *key, MapEl **lastFound )
+#if 0
+map_el_t *map_insert_key( program_t *prg, map_t *map, tree_t *key, map_el_t **last_found )
 {
-	long keyRelation;
-	MapEl *curEl = map->root, *parentEl = 0;
-	MapEl *lastLess = 0;
+	long key_relation;
+	map_el_t *cur_el = map->root, *parent_el = 0;
+	map_el_t *last_less = 0;
 
 	while ( true ) {
-		if ( curEl == 0 ) {
+		if ( cur_el == 0 ) {
 			/* We are at an external element and did not find the key we were
 			 * looking for. Create the new element, attach it underneath the leaf
 			 * and rebalance. */
-			MapEl *element = mapElAllocate( prg );
+			map_el_t *element = map_el_allocate( prg );
 			element->key = key;
-			element->tree = 0;
-			mapAttachRebal( map, element, parentEl, lastLess );
+			map_attach_rebal( map, element, parent_el, last_less );
 
-			if ( lastFound != 0 )
-				*lastFound = element;
+			if ( last_found != 0 )
+				*last_found = element;
 			return element;
 		}
 
-		keyRelation = cmpTree( prg, key, curEl->key );
+		key_relation = map_cmp( prg, map, key, cur_el->key );
 
 		/* Do we go left? */
-		if ( keyRelation < 0 ) {
-			parentEl = lastLess = curEl;
-			curEl = curEl->left;
+		if ( key_relation < 0 ) {
+			parent_el = last_less = cur_el;
+			cur_el = cur_el->left;
 		}
 		/* Do we go right? */
-		else if ( keyRelation > 0 ) {
-			parentEl = curEl;
-			curEl = curEl->right;
+		else if ( key_relation > 0 ) {
+			parent_el = cur_el;
+			cur_el = cur_el->right;
 		}
 		/* We have hit the target. */
 		else {
-			if ( lastFound != 0 )
-				*lastFound = curEl;
+			if ( last_found != 0 )
+				*last_found = cur_el;
 			return 0;
 		}
 	}
 }
+#endif
 
+map_el_t *colm_map_insert( program_t *prg, map_t *map, map_el_t *map_el )
+{
+	return map_insert_el( prg, map, map_el, 0 );
+}
+
+map_el_t *colm_vmap_insert( program_t *prg, map_t *map, struct_t *key, struct_t *value )
+{
+	struct colm_struct *s = colm_struct_new( prg, map->generic_info->el_struct_id );
+
+	colm_struct_set_field( s, struct_t*, map->generic_info->el_offset, key );
+	colm_struct_set_field( s, struct_t*, 0, value );
+
+	map_el_t *map_el = colm_struct_get_addr( s, map_el_t*, map->generic_info->el_offset );
+
+	return colm_map_insert( prg, map, map_el );
+}
+
+map_el_t *colm_vmap_remove( program_t *prg, map_t *map, tree_t *key )
+{
+	map_el_t *map_el = colm_map_find( prg, map, key );
+	if ( map_el != 0 )
+		colm_map_detach( prg, map, map_el );
+	return 0;
+}
+
+tree_t *colm_vmap_find( program_t *prg, map_t *map, tree_t *key )
+{
+	map_el_t *map_el = colm_map_find( prg, map, key );
+	if ( map_el != 0 ) {
+		struct_t *s = colm_generic_el_container( prg, map_el,
+				map->generic_info - prg->rtd->generic_info );
+		tree_t *val = colm_struct_get_field( s, tree_t*, 0 );
+
+		if ( map->generic_info->value_type == TYPE_TREE )
+			colm_tree_upref( val );
+
+		return val;
+	}
+	return 0;
+}
+
+void colm_map_detach( program_t *prg, map_t *map, map_el_t *map_el )
+{
+	map_detach( prg, map, map_el );
+}
+
+map_el_t *colm_map_find( program_t *prg, map_t *map, tree_t *key )
+{
+	return map_impl_find( prg, map, key );
+}
 
 /**
  * \brief Find a element in the tree with the given key.
  *
  * \returns The element if key exists, null if the key does not exist.
  */
-MapEl *mapImplFind( Program *prg, Map *map, Tree *key )
+map_el_t *map_impl_find( program_t *prg, map_t *map, tree_t *key )
 {
-	MapEl *curEl = map->root;
-	long keyRelation;
+	map_el_t *cur_el = map->root;
+	long key_relation;
 
-	while ( curEl != 0 ) {
-		keyRelation = cmpTree( prg, key, curEl->key );
+	while ( cur_el != 0 ) {
+		key_relation = map_cmp( prg, map, key, cur_el->key );
 
 		/* Do we go left? */
-		if ( keyRelation < 0 )
-			curEl = curEl->left;
+		if ( key_relation < 0 )
+			cur_el = cur_el->left;
 		/* Do we go right? */
-		else if ( keyRelation > 0 )
-			curEl = curEl->right;
+		else if ( key_relation > 0 )
+			cur_el = cur_el->right;
 		/* We have hit the target. */
 		else {
-			return curEl;
+			return cur_el;
 		}
 	}
 	return 0;
@@ -609,11 +738,11 @@ MapEl *mapImplFind( Program *prg, Map *map, Tree *key )
  *
  * \returns The element detached if the key is found, othewise returns null.
  */
-MapEl *mapDetachByKey( Program *prg, Map *map, Tree *key )
+map_el_t *map_detach_by_key( program_t *prg, map_t *map, tree_t *key )
 {
-	MapEl *element = mapImplFind( prg, map, key );
+	map_el_t *element = map_impl_find( prg, map, key );
 	if ( element )
-		mapDetach( prg, map, element );
+		map_detach( prg, map, element );
 
 	return element;
 }
@@ -625,16 +754,16 @@ MapEl *mapDetachByKey( Program *prg, Map *map, Tree *key )
  * 
  * \returns The element given.
  */
-MapEl *mapDetach( Program *prg, Map *map, MapEl *element )
+map_el_t *map_detach( program_t *prg, map_t *map, map_el_t *element )
 {
-	MapEl *replacement, *fixfrom;
+	map_el_t *replacement, *fixfrom;
 	long lheight, rheight;
 
 	/* Remove the element from the ordered list. */
-	mapListDetach( map, element );
+	map_list_detach( map, element );
 
 	/* Update treeSize. */
-	map->treeSize--;
+	map->tree_size--;
 
 	/* Find a replacement element. */
 	if (element->right)
@@ -652,8 +781,8 @@ MapEl *mapDetach( Program *prg, Map *map, MapEl *element )
 		else
 			fixfrom = replacement->parent;
 
-		mapRemoveEl( map, replacement, replacement->right );
-		mapReplaceEl( map, element, replacement );
+		map_remove_el( map, replacement, replacement->right );
+		map_replace_el( map, element, replacement );
 	}
 	else if (element->left)
 	{
@@ -670,8 +799,8 @@ MapEl *mapDetach( Program *prg, Map *map, MapEl *element )
 		else
 			fixfrom = replacement->parent;
 
-		mapRemoveEl( map, replacement, replacement->left );
-		mapReplaceEl( map, element, replacement );
+		map_remove_el( map, replacement, replacement->left );
+		map_replace_el( map, element, replacement );
 	}
 	else
 	{
@@ -679,7 +808,7 @@ MapEl *mapDetach( Program *prg, Map *map, MapEl *element )
 		fixfrom = element->parent;
 
 		/* The element we are deleting is a leaf element. */
-		mapRemoveEl( map, element, 0 );
+		map_remove_el( map, element, 0 );
 	}
 
 	/* If fixfrom is null it means we just deleted
@@ -688,10 +817,10 @@ MapEl *mapDetach( Program *prg, Map *map, MapEl *element )
 		return element;
 
 	/* Fix the heights after the deletion. */
-	mapRecalcHeights( map, fixfrom );
+	map_recalc_heights( map, fixfrom );
 
 	/* Fix every unbalanced element going up in the tree. */
-	MapEl *ub = mapFindFirstUnbalEl( map, fixfrom );
+	map_el_t *ub = map_find_first_unbal_el( map, fixfrom );
 	while ( ub )
 	{
 		/* Find the element to rebalance by moving down from the first unbalanced
@@ -734,10 +863,10 @@ MapEl *mapDetach( Program *prg, Map *map, MapEl *element )
 		/* rebalance returns the grandparant of the subtree formed
 		 * by the element that were rebalanced.
 		 * We must continue upward from there rebalancing. */
-		fixfrom = mapRebalance( map, ub );
+		fixfrom = map_rebalance( map, ub );
 
 		/* Find the next unbalaced element. */
-		ub = mapFindFirstUnbalEl( map, fixfrom );
+		ub = map_find_first_unbal_el( map, fixfrom );
 	}
 
 	return element;
