@@ -1,30 +1,146 @@
 /*
- *  Copyright 2006-2014 Adrian Thurston <thurston@complang.org>
+ * Copyright 2006-2014 Adrian Thurston <thurston@colm.net>
+ *
+ * Permission is hereby granted, free of charge, to any person obtaining a copy
+ * of this software and associated documentation files (the "Software"), to
+ * deal in the Software without restriction, including without limitation the
+ * rights to use, copy, modify, merge, publish, distribute, sublicense, and/or
+ * sell copies of the Software, and to permit persons to whom the Software is
+ * furnished to do so, subject to the following conditions:
+ *
+ * The above copyright notice and this permission notice shall be included in all
+ * copies or substantial portions of the Software.
+ *
+ * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
+ * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
+ * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
+ * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
+ * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
+ * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
+ * SOFTWARE.
  */
 
-#include <iostream>
+#include "parser.h"
+
+#include <stdbool.h>
+#include <stdlib.h>
 #include <errno.h>
 
-#include "parser.h"
-#include "config.h"
-#include "avltree.h"
-#include "compiler.h"
-#include "parser.h"
-#include "global.h"
-#include "input.h"
+#include <iostream>
 
 using std::endl;
 
+void BaseParser::listElDef( String name )
+{
+	/*
+	 * The unique type. This is a def with a single empty form.
+	 */
+	ObjectDef *objectDef = ObjectDef::cons( ObjectDef::UserType,
+			name, pd->nextObjectId++ );
+
+	LelDefList *defList = new LelDefList;
+
+	Production *prod = BaseParser::production( InputLoc(),
+			new ProdElList, String(), false, 0, 0 );
+
+	prodAppend( defList, prod );
+
+	NtDef *ntDef = NtDef::cons( name, curNspace(), curStruct(), false );
+	BaseParser::cflDef( ntDef, objectDef, defList );
+
+	/*
+	 * List element with the same name as containing context.
+	 */
+	NamespaceQual *nspaceQual = NamespaceQual::cons( curNspace() );
+	String id = curStruct()->objectDef->name;
+	RepeatType repeatType = RepeatNone;
+	TypeRef *objTr = TypeRef::cons( InputLoc(), nspaceQual, id, repeatType );
+	TypeRef *elTr = TypeRef::cons( InputLoc(), TypeRef::ListPtrs, 0, objTr, 0 );
+
+	ObjectField *of = ObjectField::cons( InputLoc(),
+			ObjectField::GenericElementType, elTr, name );
+
+	structVarDef( InputLoc(), of );
+}
+
+void BaseParser::mapElDef( String name, TypeRef *keyType )
+{
+	/*
+	 * The unique type. This is a def with a single empty form.
+	 */
+	ObjectDef *objectDef = ObjectDef::cons( ObjectDef::UserType,
+			name, pd->nextObjectId++ );
+
+	LelDefList *defList = new LelDefList;
+
+	Production *prod = BaseParser::production( InputLoc(),
+			new ProdElList, String(), false, 0, 0 );
+	prodAppend( defList, prod );
+
+	NtDef *ntDef = NtDef::cons( name, curNspace(), curStruct(), false );
+	BaseParser::cflDef( ntDef, objectDef, defList );
+
+	/*
+	 * Same name as containing context.
+	 */
+	NamespaceQual *nspaceQual = NamespaceQual::cons( curNspace() );
+	String id = curStruct()->objectDef->name;
+	TypeRef *objTr = TypeRef::cons( InputLoc(), nspaceQual, id, RepeatNone );
+	TypeRef *elTr = TypeRef::cons( InputLoc(), TypeRef::MapPtrs, 0, objTr, keyType );
+
+	ObjectField *of = ObjectField::cons( InputLoc(),
+			ObjectField::GenericElementType, elTr, name );
+	structVarDef( InputLoc(), of );
+}
+
+#if 0
+void BaseParser::argvDecl()
+{
+	String structName = "argv_el";
+	structHead( internal, pd->rootNamespace, structName, ObjectDef::StructType );
+
+	/* First the argv value. */
+	String name = "value";
+	String type = "str";
+	NamespaceQual *nspaceQual = NamespaceQual::cons( curNspace() );
+	TypeRef *typeRef = TypeRef::cons( internal, nspaceQual, type, RepeatNone );
+	ObjectField *objField = ObjectField::cons( internal,
+			ObjectField::StructFieldType, typeRef, name );
+	structVarDef( objField->loc, objField );
+
+	pd->argvEl = objField->context;
+
+	/* Now the list element. */
+	listElDef( "el" );
+
+	structStack.pop();
+	namespaceStack.pop();
+}
+#endif
+
 void BaseParser::init()
 {
 	/* Set up the root namespace. */
-	Namespace *rootNamespace = createRootNamespace();
-	pd->rootNamespace = rootNamespace;
+	pd->rootNamespace = createRootNamespace();
 
-	/* Set up the global object. */
+	/* Setup the global object. */
 	String global = "global";
 	pd->globalObjectDef = ObjectDef::cons( ObjectDef::UserType,
 			global, pd->nextObjectId++ ); 
+
+	pd->rootNamespace->rootScope->owningObj = pd->globalObjectDef;
+
+	pd->global = new StructDef( internal, global, pd->globalObjectDef );
+	pd->globalSel = declareStruct( pd, 0, global, pd->global );
+
+	/* Setup the stream object. */
+	global = "stream";
+	ObjectDef *objectDef = ObjectDef::cons( ObjectDef::BuiltinType,
+			global, pd->nextObjectId++ ); 
+
+	pd->stream = new StructDef( internal, global, objectDef );
+	pd->streamSel = declareStruct( pd, pd->rootNamespace,
+			pd->stream->name, pd->stream );
 	
 	/* Initialize the dictionary of graphs. This is our symbol table. The
 	 * initialization needs to be done on construction which happens at the
@@ -34,14 +150,17 @@ void BaseParser::init()
 
 	pd->rootLocalFrame = ObjectDef::cons( ObjectDef::FrameType, 
 				"local", pd->nextObjectId++ );
-	curLocalFrame = pd->rootLocalFrame;
-	curScope = pd->rootLocalFrame->rootScope;
+	localFrameTop = pd->rootLocalFrame;
+	scopeTop = pd->rootLocalFrame->rootScope;
+
 
 	/* Declarations of internal types. They must be declared now because we use
 	 * them directly, rather than via type lookup. */
 	pd->declareBaseLangEls();
 	pd->initUniqueTypes();
 
+	//argvDecl();
+
 	/* Internal variables. */
 	addArgvList();
 }
@@ -108,6 +227,8 @@ Namespace *BaseParser::createRootNamespace()
 	Namespace *nspace = new Namespace( internal,
 			String("___ROOT_NAMESPACE"), 0, 0 );
 
+	nspace->rootScope->owningObj = pd->globalObjectDef;
+
 	pd->namespaceList.append( nspace );
 	namespaceStack.push( nspace );
 
@@ -125,6 +246,10 @@ Namespace *BaseParser::createNamespace( const InputLoc &loc, const String &name
 		nspace = new Namespace( loc, name,
 				pd->namespaceList.length(), parent );
 
+		/* Link the new namespace's scope to the parent namespace's scope. */
+		nspace->rootScope->parentScope = parent->rootScope;
+		nspace->rootScope->owningObj = pd->globalObjectDef;
+
 		parent->childNamespaces.append( nspace );
 		pd->namespaceList.append( nspace );
 	}
@@ -134,6 +259,21 @@ Namespace *BaseParser::createNamespace( const InputLoc &loc, const String &name
 	return nspace;
 }
 
+Reduction *BaseParser::createReduction( const InputLoc loc, const String &name )
+{
+	Namespace *parent = namespaceStack.top();
+	Reduction *reduction = parent->findReduction( name );
+
+	if ( reduction == 0 ) {
+		reduction = new Reduction( loc, name );
+		parent->reductions.append( reduction );
+	}
+
+	reductionStack.push( reduction );
+
+	return reduction;
+}
+
 LexJoin *BaseParser::literalJoin( const InputLoc &loc, const String &data )
 {
 	Literal *literal = Literal::cons( loc, data, Literal::LitString );
@@ -147,8 +287,9 @@ LexJoin *BaseParser::literalJoin( const InputLoc &loc, const String &data )
 	return join;
 }
 
-void BaseParser::defineToken( const InputLoc &loc, String name, LexJoin *join, ObjectDef *objectDef,
-		CodeBlock *transBlock, bool ignore, bool noPreIgnore, bool noPostIgnore )
+void BaseParser::defineToken( const InputLoc &loc, String name, LexJoin *join,
+		ObjectDef *objectDef, CodeBlock *transBlock, bool ignore,
+		bool noPreIgnore, bool noPostIgnore )
 {
 	bool pushedRegion = false;
 	if ( !insideRegion() ) {
@@ -171,7 +312,7 @@ void BaseParser::defineToken( const InputLoc &loc, String name, LexJoin *join, O
 	RegionSet *regionSet = regionStack.top();
 
 	TokenDef *tokenDef = TokenDef::cons( name, String(), false, ignore, join, 
-			transBlock, loc, 0, nspace, regionSet, objectDef, curContext() );
+			transBlock, loc, 0, nspace, regionSet, objectDef, curStruct() );
 
 	regionSet->tokenDefList.append( tokenDef );
 	nspace->tokenDefList.append( tokenDef );
@@ -229,7 +370,7 @@ void BaseParser::zeroDef( const InputLoc &loc, const String &name )
 	LexJoin *join = literalJoin( loc, String("`") );
 
 	TokenDef *tokenDef = TokenDef::cons( name, String(), false, false, join,
-			0, loc, 0, nspace, regionSet, 0, curContext() );
+			0, loc, 0, nspace, regionSet, 0, curStruct() );
 
 	tokenDef->isZero = true;
 
@@ -302,8 +443,9 @@ void BaseParser::literalDef( const InputLoc &loc, const String &data,
 
 void BaseParser::addArgvList()
 {
-	TypeRef *typeRef = TypeRef::cons( internal, pd->uniqueTypeStr );
-	pd->argvTypeRef = TypeRef::cons( internal, TypeRef::List, 0, typeRef, 0 );
+	TypeRef *valType = TypeRef::cons( internal, pd->uniqueTypeStr );
+	TypeRef *elType = TypeRef::cons( internal, TypeRef::ListEl, valType );
+	pd->argvTypeRef = TypeRef::cons( internal, TypeRef::List, 0, elType, valType );
 }
 
 ObjectDef *BaseParser::blockOpen()
@@ -312,32 +454,44 @@ ObjectDef *BaseParser::blockOpen()
 	ObjectDef *frame = ObjectDef::cons( ObjectDef::FrameType, 
 			"local", pd->nextObjectId++ );
 
-	curLocalFrame = frame;
-	curScope = frame->rootScope;
+	localFrameTop = frame;
+	scopeTop = frame->rootScope;
 	return frame;
 }
 
 void BaseParser::blockClose()
 {
-	curLocalFrame = pd->rootLocalFrame;
-	curScope = pd->rootLocalFrame->rootScope;
+	localFrameTop = pd->rootLocalFrame;
+	scopeTop = pd->rootLocalFrame->rootScope;
 }
 
 void BaseParser::functionDef( StmtList *stmtList, ObjectDef *localFrame,
 	ParameterList *paramList, TypeRef *typeRef, const String &name, bool exprt )
 {
 	CodeBlock *codeBlock = CodeBlock::cons( stmtList, localFrame );
-	Function *newFunction = Function::cons( typeRef, name, 
+	Function *newFunction = Function::cons( curNspace(), typeRef, name, 
 			paramList, codeBlock, pd->nextFuncId++, false, exprt );
 	pd->functionList.append( newFunction );
-	newFunction->inContext = curContext();
+	newFunction->inContext = curStruct();
+}
+
+void BaseParser::inHostDef( const String &hostCall, ObjectDef *localFrame,
+	ParameterList *paramList, TypeRef *typeRef, const String &name, bool exprt )
+{
+	Function *newFunction = Function::cons( curNspace(), typeRef, name, 
+			paramList, 0, pd->nextHostId++, false, exprt );
+	newFunction->hostCall = hostCall;
+	newFunction->localFrame = localFrame;
+	newFunction->inHost = true;
+	pd->inHostList.append( newFunction );
+	newFunction->inContext = curStruct();
 }
 
 void BaseParser::iterDef( StmtList *stmtList, ObjectDef *localFrame,
 	ParameterList *paramList, const String &name )
 {
 	CodeBlock *codeBlock = CodeBlock::cons( stmtList, localFrame );
-	Function *newFunction = Function::cons( 0, name, 
+	Function *newFunction = Function::cons( curNspace(), 0, name, 
 			paramList, codeBlock, pd->nextFuncId++, true, false );
 	pd->functionList.append( newFunction );
 }
@@ -346,33 +500,52 @@ LangStmt *BaseParser::globalDef( ObjectField *objField, LangExpr *expr,
 		LangStmt::Type assignType )
 {
 	LangStmt *stmt = 0;
+	ObjectDef *object = pd->globalObjectDef;
+	Namespace *nspace = curNspace(); //pd->rootNamespace;
 
-	Context *context = 0;
-	ObjectDef *object = 0;
-	if ( curContext() == 0 )
-		object = pd->globalObjectDef;
-	else {
-		context = curContext();
-		objField->context = context;
-		object = context->objectDef;
-	}
-
-	if ( object->rootScope->checkRedecl( objField->name ) != 0 )
+	if ( nspace->rootScope->checkRedecl( objField->name ) != 0 )
 		error(objField->loc) << "object field renamed" << endp;
 
-	object->rootScope->insertField( objField->name, objField );
+	object->insertField( nspace->rootScope, objField->name, objField );
 
 	if ( expr != 0 ) {
 		LangVarRef *varRef = LangVarRef::cons( objField->loc,
-				context, curScope, objField->name );
+				curNspace(), curStruct(), curScope(), objField->name );
 
-		stmt = LangStmt::cons( objField->loc, 
-				assignType, varRef, expr );
+		stmt = LangStmt::cons( objField->loc, assignType, varRef, expr );
 	}
 
 	return stmt;
 }
 
+LangStmt *BaseParser::exportStmt( ObjectField *objField,
+		LangStmt::Type assignType, LangExpr *expr )
+{
+	LangStmt *stmt = 0;
+
+	ObjectDef *object = pd->globalObjectDef;
+	Namespace *nspace = curNspace(); //pd->rootNamespace;
+
+	if ( curStruct() != 0 )
+		error(objField->loc) << "cannot export parser context variables" << endp;
+
+	if ( nspace->rootScope->checkRedecl( objField->name ) != 0 )
+		error(objField->loc) << "object field renamed" << endp;
+
+	object->insertField( nspace->rootScope, objField->name, objField );
+	objField->isExport = true;
+
+	if ( expr != 0 ) {
+		LangVarRef *varRef = LangVarRef::cons( objField->loc, 
+				curNspace(), 0, curScope(), objField->name );
+
+		stmt = LangStmt::cons( objField->loc, assignType, varRef, expr );
+	}
+
+	return stmt;
+}
+
+
 void BaseParser::cflDef( NtDef *ntDef, ObjectDef *objectDef, LelDefList *defList )
 {
 	Namespace *nspace = curNspace();
@@ -468,7 +641,7 @@ int BaseParser::lexFactorRepNum( const InputLoc &loc, const String &data )
 {
 	/* Convert the priority number to a long. Check for overflow. */
 	errno = 0;
-	int rep = strtol( data, 0, 10 );
+	long rep = strtol( data, 0, 10 );
 	if ( errno == ERANGE && rep == LONG_MAX ) {
 		/* Repetition too large. Recover by returing repetition 1. */
 		error(loc) << "repetition number " << data << " overflows" << endl;
@@ -517,7 +690,8 @@ LexJoin *BaseParser::lexOptJoin( LexJoin *join, LexJoin *context )
 LangExpr *BaseParser::send( const InputLoc &loc, LangVarRef *varRef,
 		ConsItemList *list, bool eof )
 {
-	ParserText *parserText = ParserText::cons( loc, curNspace(), list );
+	ParserText *parserText = ParserText::cons( loc,
+			curNspace(), list, true, false, "" );
 	pd->parserTextList.append( parserText );
 
 	return LangExpr::cons( LangTerm::consSend( loc, varRef,
@@ -527,7 +701,8 @@ LangExpr *BaseParser::send( const InputLoc &loc, LangVarRef *varRef,
 LangExpr *BaseParser::sendTree( const InputLoc &loc, LangVarRef *varRef,
 		ConsItemList *list, bool eof )
 {
-	ParserText *parserText = ParserText::cons( loc, curNspace(), list );
+	ParserText *parserText = ParserText::cons( loc,
+			curNspace(), list, true, false, "" );
 	pd->parserTextList.append( parserText );
 
 	return LangExpr::cons( LangTerm::consSendTree( loc, varRef,
@@ -536,49 +711,48 @@ LangExpr *BaseParser::sendTree( const InputLoc &loc, LangVarRef *varRef,
 
 LangExpr *BaseParser::parseCmd( const InputLoc &loc, bool tree, bool stop,
 		ObjectField *objField, TypeRef *typeRef, FieldInitVect *fieldInitVect,
-		ConsItemList *list )
+		ConsItemList *list, bool used, bool reduce, const String &reducer )
 {
 	LangExpr *expr = 0;
 
-	/* We are constructing a parser, sending it items, then returning it.
-	 * Thisis the constructor for the parser. */
-	ConsItemList *emptyConsItemList = new ConsItemList;
-
-	Constructor *constructor = Constructor::cons( loc, curNspace(),
-			emptyConsItemList, pd->nextPatConsId++ );
-	pd->replList.append( constructor );
+	/* Item list for what we are sending to the parser. */
+	ConsItemList *consItemList = new ConsItemList;
 	
 	/* The parser may be referenced. */
 	LangVarRef *varRef = 0;
 	if ( objField != 0 ) {
 		varRef = LangVarRef::cons( objField->loc,
-				curContext(), curScope, objField->name );
+				curNspace(), curStruct(), curScope(), objField->name );
 	}
 
 	/* The typeref for the parser. */
 	TypeRef *parserTypeRef = TypeRef::cons( loc,
 			TypeRef::Parser, 0, typeRef, 0 );
 
-	ParserText *parserText = ParserText::cons( loc, curNspace(), list );
+	if ( objField != 0 )
+		used = true;
+
+	ParserText *parserText = ParserText::cons( loc, curNspace(),
+			list, used, reduce, reducer );
 	pd->parserTextList.append( parserText );
 
 	LangTerm::Type langTermType = stop ? LangTerm::ParseStopType : ( tree ? 
 					LangTerm::ParseTreeType : LangTerm::ParseType );
 
 	expr = LangExpr::cons( LangTerm::cons( loc, langTermType,
-			varRef, objField, parserTypeRef, fieldInitVect, constructor,
+			varRef, objField, parserTypeRef, fieldInitVect, consItemList,
 			parserText ) );
 
 	/* Check for redeclaration. */
 	if ( objField != 0 ) {
-		if ( curScope->checkRedecl( objField->name ) != 0 ) {
+		if ( curScope()->checkRedecl( objField->name ) != 0 ) {
 			error( objField->loc ) << "variable " << objField->name <<
 					" redeclared" << endp;
 		}
 
 		/* Insert it into the field map. */
 		objField->typeRef = typeRef;
-		curScope->insertField( objField->name, objField );
+		curScope()->insertField( objField->name, objField );
 	}
 
 	return expr;
@@ -590,7 +764,7 @@ PatternItemList *BaseParser::consPatternEl( LangVarRef *varRef, PatternItemList
 	list->head->varRef = varRef;
 
 	if ( varRef != 0 ) {
-		if ( curScope->checkRedecl( varRef->name ) != 0 ) {
+		if ( curScope()->checkRedecl( varRef->name ) != 0 ) {
 			error( varRef->loc ) << "variable " << varRef->name << 
 					" redeclared" << endp;
 		}
@@ -600,7 +774,7 @@ PatternItemList *BaseParser::consPatternEl( LangVarRef *varRef, PatternItemList
 				ObjectField::UserLocalType, typeRef, varRef->name );
 
 		/* Insert it into the field map. */
-		curScope->insertField( varRef->name, objField );
+		curScope()->insertField( varRef->name, objField );
 	}
 
 	return list;
@@ -655,7 +829,7 @@ LangStmt *BaseParser::forScope( const InputLoc &loc, const String &data,
 		NameScope *scope, TypeRef *typeRef, IterCall *iterCall, StmtList *stmtList )
 {
 	/* Check for redeclaration. */
-	if ( curScope->checkRedecl( data ) != 0 )
+	if ( curScope()->checkRedecl( data ) != 0 )
 		error( loc ) << "variable " << data << " redeclared" << endp;
 
 	/* Note that we pass in a null type reference. This type is dependent on
@@ -663,10 +837,10 @@ LangStmt *BaseParser::forScope( const InputLoc &loc, const String &data,
 	 * the iterator that is called. This lookup is done at compile time. */
 	ObjectField *iterField = ObjectField::cons( loc,
 			ObjectField::UserLocalType, (TypeRef*)0, data );
-	curScope->insertField( data, iterField );
+	curScope()->insertField( data, iterField );
 
 	LangStmt *stmt = LangStmt::cons( loc, LangStmt::ForIterType, 
-			iterField, typeRef, iterCall, stmtList, curContext(), scope );
+			iterField, typeRef, iterCall, stmtList, curStruct(), scope );
 
 	return stmt;
 }
@@ -677,7 +851,7 @@ void BaseParser::preEof( const InputLoc &loc, StmtList *stmtList, ObjectDef *loc
 		error(loc) << "preeof must be used inside an existing region" << endl;
 
 	CodeBlock *codeBlock = CodeBlock::cons( stmtList, localFrame );
-	codeBlock->context = curContext();
+	codeBlock->context = curStruct();
 
 	RegionSet *regionSet = regionStack.top();
 	regionSet->tokenIgnore->preEofBlock = codeBlock;
@@ -755,7 +929,7 @@ LangExpr *BaseParser::construct( const InputLoc &loc, ObjectField *objField,
 	LangVarRef *varRef = 0;
 	if ( objField != 0 ) {
 		varRef = LangVarRef::cons( objField->loc,
-				curContext(), curScope, objField->name );
+				curNspace(), curStruct(), curScope(), objField->name );
 	}
 
 	LangExpr *expr = LangExpr::cons( LangTerm::cons( loc, LangTerm::ConstructType,
@@ -763,14 +937,14 @@ LangExpr *BaseParser::construct( const InputLoc &loc, ObjectField *objField,
 
 	/* Check for redeclaration. */
 	if ( objField != 0 ) {
-		if ( curScope->checkRedecl( objField->name ) != 0 ) {
+		if ( curScope()->checkRedecl( objField->name ) != 0 ) {
 			error( objField->loc ) << "variable " << objField->name <<
 					" redeclared" << endp;
 		}
 
 		/* Insert it into the field map. */
 		objField->typeRef = typeRef;
-		curScope->insertField( objField->name, objField );
+		curScope()->insertField( objField->name, objField );
 	}
 
 	return expr;
@@ -795,45 +969,19 @@ LangStmt *BaseParser::varDef( ObjectField *objField,
 	LangStmt *stmt = 0;
 
 	/* Check for redeclaration. */
-	if ( curScope->checkRedecl( objField->name ) != 0 ) {
+	if ( curScope()->checkRedecl( objField->name ) != 0 ) {
 		error( objField->loc ) << "variable " << objField->name <<
 				" redeclared" << endp;
 	}
 
 	/* Insert it into the field map. */
-	curScope->insertField( objField->name, objField );
+	curScope()->insertField( objField->name, objField );
 
 	//cout << "var def " << $1->objField->name << endl;
 
 	if ( expr != 0 ) {
 		LangVarRef *varRef = LangVarRef::cons( objField->loc,
-				curContext(), curScope, objField->name );
-
-		stmt = LangStmt::cons( objField->loc, assignType, varRef, expr );
-	}
-
-	return stmt;
-}
-
-LangStmt *BaseParser::exportStmt( ObjectField *objField,
-		LangStmt::Type assignType, LangExpr *expr )
-{
-	LangStmt *stmt = 0;
-
-	if ( curContext() != 0 )
-		error(objField->loc) << "cannot export parser context variables" << endp;
-
-	ObjectDef *object = pd->globalObjectDef;
-
-	if ( object->rootScope->checkRedecl( objField->name ) != 0 )
-		error(objField->loc) << "object field renamed" << endp;
-
-	object->rootScope->insertField( objField->name, objField );
-	objField->isExport = true;
-
-	if ( expr != 0 ) {
-		LangVarRef *varRef = LangVarRef::cons( objField->loc, 
-				0, curScope, objField->name );
+				curNspace(), curStruct(), curScope(), objField->name );
 
 		stmt = LangStmt::cons( objField->loc, assignType, varRef, expr );
 	}
@@ -853,15 +1001,14 @@ LangExpr *BaseParser::require( const InputLoc &loc,
 	return expr;
 }
 
-void BaseParser::contextVarDef( const InputLoc &loc, ObjectField *objField )
+void BaseParser::structVarDef( const InputLoc &loc, ObjectField *objField )
 {
 	ObjectDef *object;
-	if ( curContext() == 0 )
+	if ( curStruct() == 0 )
 		error(loc) << "internal error: no context stack items found" << endp;
 
-	Context *context = curContext();
-	objField->context = context;
-	object = context->objectDef;
+	StructDef *structDef = curStruct();
+	object = structDef->objectDef;
 
 	if ( object->rootScope->checkRedecl( objField->name ) != 0 )
 		error(objField->loc) << "object field renamed" << endp;
@@ -869,20 +1016,19 @@ void BaseParser::contextVarDef( const InputLoc &loc, ObjectField *objField )
 	object->rootScope->insertField( objField->name, objField );
 }
 
-void BaseParser::contextHead( const InputLoc &loc, const String &data,
-		ObjectDef::Type objectType )
+void BaseParser::structHead( const InputLoc &loc, Namespace *inNspace,
+		const String &data, ObjectDef::Type objectType )
 {
-	/* Make the new namespace. */
-	Namespace *nspace = createNamespace( loc, data );
-
-	Context *context = new Context( loc, 0 );
-	contextStack.push( context );
-
-	ContextDef *contextDef = new ContextDef( data, context, nspace );
-	nspace->contextDefList.append( contextDef );
-
-	context->objectDef = ObjectDef::cons( objectType,
+	ObjectDef *objectDef = ObjectDef::cons( objectType,
 			data, pd->nextObjectId++ ); 
+
+	StructDef *context = new StructDef( loc, data, objectDef );
+	structStack.push( context );
+
+	inNspace->structDefList.append( context );
+
+	/* Make the namespace for the struct. */
+	createNamespace( loc, data );
 }
 
 StmtList *BaseParser::appendStatement( StmtList *stmtList, LangStmt *stmt )
@@ -941,10 +1087,10 @@ void BaseParser::precedenceStmt( PredType predType, PredDeclList *predDeclList )
 
 void BaseParser::pushScope()
 {
-	curScope = curLocalFrame->pushScope( curScope );
+	scopeTop = curLocalFrame()->pushScope( curScope() );
 }
 
 void BaseParser::popScope()
 {
-	curScope = curScope->parentScope;
+	scopeTop = curScope()->parentScope;
 }
