@@ -1,19 +1,33 @@
 /*
- *  Copyright 2006-2012 Adrian Thurston <thurston@complang.org>
+ * Copyright 2006-2012 Adrian Thurston <thurston@colm.net>
+ *
+ * Permission is hereby granted, free of charge, to any person obtaining a copy
+ * of this software and associated documentation files (the "Software"), to
+ * deal in the Software without restriction, including without limitation the
+ * rights to use, copy, modify, merge, publish, distribute, sublicense, and/or
+ * sell copies of the Software, and to permit persons to whom the Software is
+ * furnished to do so, subject to the following conditions:
+ *
+ * The above copyright notice and this permission notice shall be included in all
+ * copies or substantial portions of the Software.
+ *
+ * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
+ * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
+ * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
+ * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
+ * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
+ * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
+ * SOFTWARE.
  */
 
-#include "avltree.h"
-#include "compiler.h"
-#include "parser.h"
-#include "parsetree.h"
-#include "input.h"
+#include <assert.h>
+#include <stdbool.h>
 
 #include <iostream>
-#include <iomanip>
-#include <errno.h>
-#include <limits.h>
-#include <stdlib.h>
 
+#include "fsmgraph.h"
+#include "compiler.h"
+#include "parsetree.h"
 
 using namespace std;
 ostream &operator<<( ostream &out, const NameRef &nameRef );
@@ -89,23 +103,38 @@ int CmpUniqueType::compare( const UniqueType &ut1, const UniqueType &ut2 )
 		return 1;
 	switch ( ut1.typeId ) {
 		case TYPE_TREE:
-		case TYPE_PTR:
 		case TYPE_REF:
-		{
 			if ( ut1.langEl < ut2.langEl )
 				return -1;
 			else if ( ut1.langEl > ut2.langEl )
 				return 1;
 			break;
-		}
-		case TYPE_ITER: {
+		case TYPE_ITER:
 			if ( ut1.iterDef < ut2.iterDef )
 				return -1;
 			else if ( ut1.iterDef > ut2.iterDef )
 				return 1;
 			break;
-		}
+
+		case TYPE_NOTYPE:
 		case TYPE_NIL:
+		case TYPE_INT:
+		case TYPE_BOOL:
+		case TYPE_LIST_PTRS:
+		case TYPE_MAP_PTRS:
+			break;
+
+		case TYPE_STRUCT:
+			if ( ut1.structEl < ut2.structEl )
+				return -1;
+			else if ( ut1.structEl > ut2.structEl )
+				return 1;
+			break;
+		case TYPE_GENERIC:
+			if ( ut1.generic < ut2.generic )
+				return -1;
+			else if ( ut1.generic > ut2.generic )
+				return 1;
 			break;
 	}
 
@@ -128,95 +157,32 @@ int CmpUniqueRepeat::compare( const UniqueRepeat &ut1, const UniqueRepeat &ut2 )
 	return 0;
 }
 
-int CmpUniqueMap::compare( const UniqueMap &ut1, const UniqueMap &ut2 )
+int CmpUniqueGeneric::compare( const UniqueGeneric &ut1, const UniqueGeneric &ut2 )
 {
-	if ( ut1.key < ut2.key )
+	if ( ut1.type < ut2.type )
 		return -1;
-	else if ( ut1.key > ut2.key )
+	else if ( ut1.type > ut2.type )
 		return 1;
-	else {
-		if ( ut1.value < ut2.value )
-			return -1;
-		else if ( ut1.value > ut2.value )
-			return 1;
-	}
-
-	return 0;
-}
-
-int CmpUniqueList::compare( const UniqueList &ut1, const UniqueList &ut2 )
-{
-	if ( ut1.value < ut2.value )
-		return -1;
-	else if ( ut1.value > ut2.value )
-		return 1;
-
-	return 0;
-}
-
-int CmpUniqueList2El::compare( const UniqueList2El &ut1, const UniqueList2El &ut2 )
-{
-	if ( ut1.value < ut2.value )
-		return -1;
-	else if ( ut1.value > ut2.value )
-		return 1;
-
-	return 0;
-}
-
-int CmpUniqueList2::compare( const UniqueList2 &ut1, const UniqueList2 &ut2 )
-{
-	if ( ut1.value < ut2.value )
+	else if ( ut1.value < ut2.value )
 		return -1;
 	else if ( ut1.value > ut2.value )
 		return 1;
 	else {
-		if ( ut1.attrOff < ut2.attrOff )
-			return -1;
-		else if ( ut1.attrOff > ut2.attrOff )
-			return 1;
+		switch ( ut1.type ) {
+		case UniqueGeneric::List:
+		case UniqueGeneric::ListEl:
+		case UniqueGeneric::Parser:
+			break;
+
+		case UniqueGeneric::Map:
+		case UniqueGeneric::MapEl:
+			if ( ut1.key < ut2.key )
+				return -1;
+			else if ( ut1.key > ut2.key )
+				return 1;
+			break;
+		}
 	}
-
-	return 0;
-}
-
-int CmpUniqueMap2El::compare( const UniqueMap2El &ut1, const UniqueMap2El &ut2 )
-{
-	if ( ut1.value < ut2.value )
-		return -1;
-	else if ( ut1.value > ut2.value )
-		return 1;
-
-	return 0;
-}
-
-int CmpUniqueMap2::compare( const UniqueMap2 &ut1, const UniqueMap2 &ut2 )
-{
-	if ( ut1.value < ut2.value )
-		return -1;
-	else if ( ut1.value > ut2.value )
-		return 1;
-
-	return 0;
-}
-
-int CmpUniqueVector::compare( const UniqueVector &ut1, const UniqueVector &ut2 )
-{
-	if ( ut1.value < ut2.value )
-		return -1;
-	else if ( ut1.value > ut2.value )
-		return 1;
-
-	return 0;
-}
-
-int CmpUniqueParser::compare( const UniqueParser &ut1, const UniqueParser &ut2 )
-{
-	if ( ut1.parseType < ut2.parseType )
-		return -1;
-	else if ( ut1.parseType > ut2.parseType )
-		return 1;
-
 	return 0;
 }
 
@@ -1393,6 +1359,10 @@ FsmGraph *ReItem::walk( Compiler *pd, RegExpr *rootRegex )
 		case OrBlock: {
 			/* Get the or block and minmize it. */
 			rtnVal = orBlock->walk( pd, rootRegex );
+			if ( rtnVal == 0 ) {
+				rtnVal = new FsmGraph();
+				rtnVal->lambdaFsm();
+			}
 			rtnVal->minimizePartition2();
 			break;
 		}
