@@ -1,24 +1,45 @@
 /*
- *  Copyright 2006-2012 Adrian Thurston <thurston@complang.org>
+ * Copyright 2006-2012 Adrian Thurston <thurston@colm.net>
+ *
+ * Permission is hereby granted, free of charge, to any person obtaining a copy
+ * of this software and associated documentation files (the "Software"), to
+ * deal in the Software without restriction, including without limitation the
+ * rights to use, copy, modify, merge, publish, distribute, sublicense, and/or
+ * sell copies of the Software, and to permit persons to whom the Software is
+ * furnished to do so, subject to the following conditions:
+ *
+ * The above copyright notice and this permission notice shall be included in all
+ * copies or substantial portions of the Software.
+ *
+ * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
+ * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
+ * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
+ * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
+ * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
+ * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
+ * SOFTWARE.
  */
 
-#ifndef _PARSETREE_H
-#define _PARSETREE_H
+#ifndef _COLM_PARSETREE_H
+#define _COLM_PARSETREE_H
 
 #include <iostream>
 #include <string.h>
+#include <string>
+
+#include <avlbasic.h>
+#include <avlmap.h>
+#include <bstmap.h>
+#include <bstset.h>
+#include <vector.h>
+#include <dlist.h>
+#include <dlistval.h>
+#include <dlistmel.h>
 
 #include "global.h"
-#include "avlmap.h"
-#include "bstmap.h"
-#include "bstset.h"
-#include "vector.h"
-#include "dlist.h"
-#include "dlistval.h"
-#include "dlistmel.h"
 #include "cstring.h"
 #include "bytecode.h"
-#include "avlbasic.h"
+
 
 /* Operators that are represented with single symbol characters. */
 #define OP_DoubleEql 'e'
@@ -36,7 +57,6 @@
 struct NameInst;
 struct FsmGraph;
 struct RedFsm;
-struct _FsmRun;
 struct ObjectDef;
 struct ElementOf;
 struct UniqueType;
@@ -48,21 +68,25 @@ struct TypeAlias;
 struct RegionSet;
 struct NameScope;
 struct IterCall;
-typedef struct _PdaRun PdaRun;
+struct TemplateType;
+struct ObjectMethod;
+struct Reduction;
+struct Production;
+
 
 /* 
  * Code Vector
  */
-struct CodeVect : public Vector<Code>
+struct CodeVect : public Vector<code_t>
 {
-	void appendHalf( Half half )
+	void appendHalf( half_t half )
 	{
 		/* not optimal. */
 		append( half & 0xff );
 		append( (half>>8) & 0xff );
 	}
 	
-	void appendWord( Word word )
+	void appendWord( word_t word )
 	{
 		/* not optimal. */
 		append( word & 0xff );
@@ -77,21 +101,21 @@ struct CodeVect : public Vector<Code>
 		#endif
 	}
 
-	void setHalf( long pos, Half half )
+	void setHalf( long pos, half_t half )
 	{
 		/* not optimal. */
 		data[pos] = half & 0xff;
 		data[pos+1] = (half>>8) & 0xff;
 	}
 	
-	void insertHalf( long pos, Half half )
+	void insertHalf( long pos, half_t half )
 	{
 		/* not optimal. */
 		insert( pos, half & 0xff );
 		insert( pos+1, (half>>8) & 0xff );
 	}
 
-	void insertWord( long pos, Word word )
+	void insertWord( long pos, word_t word )
 	{
 		/* not at all optimal. */
 		insert( pos, word & 0xff );
@@ -106,8 +130,8 @@ struct CodeVect : public Vector<Code>
 		#endif
 	}
 	
-	void insertTree( long pos, Tree *tree )
-		{ insertWord( pos, (Word) tree ); }
+	void insertTree( long pos, tree_t *tree )
+		{ insertWord( pos, (word_t) tree ); }
 };
 
 
@@ -174,7 +198,7 @@ struct TypeRef;
 /* Leaf type. */
 struct Literal;
 
-/* Tree nodes. */
+/* tree_t nodes. */
 
 struct LexTerm;
 struct LexFactorAug;
@@ -187,7 +211,7 @@ struct JoinOrLm;
 struct RegionJoinOrLm;
 struct TokenRegion;
 struct Namespace;
-struct Context;
+struct StructDef;
 struct TokenDef;
 struct TokenDefListReg;
 struct TokenDefListNs;
@@ -199,7 +223,7 @@ struct LangEl;
 enum AugType
 {
 	at_start,
-	at_leave,
+	at_leave
 };
 
 struct Action;
@@ -330,27 +354,6 @@ struct ReCapture
 	ObjectField *objField;
 };
 
-struct ContextStack
-	: public Vector<Context*>
-{
-	Context *top()
-		{ return length() > 0 ? Vector<Context*>::top() : 0; }
-};
-
-
-struct Context
-{
-	Context( const InputLoc &loc, LangEl *lel )
-	:
-		loc(loc),
-		lel(lel),
-		objectDef(0)
-	{}
-
-	InputLoc loc;
-	LangEl *lel;
-	ObjectDef *objectDef;
-};
 
 typedef Vector<ReCapture> ReCaptureVect;
 
@@ -378,7 +381,7 @@ struct TokenDef
 	static TokenDef *cons( const String &name, const String &literal,
 		bool isLiteral, bool isIgnore, LexJoin *join, CodeBlock *codeBlock,
 		const InputLoc &semiLoc, int longestMatchId, Namespace *nspace,
-		RegionSet *regionSet, ObjectDef *objectDef, Context *contextIn )
+		RegionSet *regionSet, ObjectDef *objectDef, StructDef *contextIn )
 	{ 
 		TokenDef *t = new TokenDef;
 
@@ -427,7 +430,7 @@ struct TokenDef
 	RegionSet *regionSet;
 	ReCaptureVect reCaptureVect;
 	ObjectDef *objectDef;
-	Context *contextIn;
+	StructDef *contextIn;
 
 	TokenDef *dupOf;
 	bool noPostIgnore;
@@ -495,7 +498,7 @@ struct NtDef
 {
 	static NtDef *cons( const String &name, Namespace *nspace,
 		LelDefList *defList, ObjectDef *objectDef,
-		Context *contextIn, bool reduceFirst )
+		StructDef *contextIn, bool reduceFirst )
 	{ 
 		NtDef *nt = new NtDef;
 
@@ -510,7 +513,7 @@ struct NtDef
 	}
 
 	static NtDef *cons( const String &name, Namespace *nspace,
-		Context *contextIn, bool reduceFirst )
+		StructDef *contextIn, bool reduceFirst )
 	{ 
 		NtDef *nt = new NtDef;
 
@@ -528,7 +531,7 @@ struct NtDef
 	Namespace *nspace;
 	LelDefList *defList;
 	ObjectDef *objectDef;
-	Context *contextIn;
+	StructDef *contextIn;
 	bool reduceFirst;
 
 	NtDef *prev, *next;
@@ -543,19 +546,51 @@ struct TokenInstanceListReg : DListMel<TokenInstance, TokenInstancePtr> {};
 struct TokenDefListReg : DListMel<TokenDef, TokenDefPtr1> {};
 struct TokenDefListNs : DListMel<TokenDef, TokenDefPtr2> {};
 
-struct ContextDef
+struct StructStack
+	: public Vector<StructDef*>
 {
-	ContextDef( const String &name, Context *context, Namespace *nspace )
-		: name(name), context(context), nspace(nspace) {}
-
-	String name;
-	Context *context;
-	Namespace *nspace;
-
-	ContextDef *prev, *next;
+	StructDef *top()
+		{ return length() > 0 ? Vector<StructDef*>::top() : 0; }
 };
 
-struct ContextDefList : DList<ContextDef> {};
+struct StructEl;
+
+struct StructDef
+{
+	StructDef( const InputLoc &loc, const String &name, ObjectDef *objectDef )
+	:
+		loc(loc),
+		name(name),
+		objectDef(objectDef),
+		structEl(0)
+	{}
+
+	InputLoc loc;
+	String name;
+	ObjectDef *objectDef;
+	StructEl *structEl;
+
+	StructDef *prev, *next;
+};
+
+struct StructEl
+{
+	StructEl( const String &name, StructDef *structDef )
+	:
+		name(name),
+		structDef(structDef),
+		id(-1)
+	{}
+
+	String name;
+	StructDef *structDef;
+	int id;
+
+	StructEl *prev, *next;
+};
+
+typedef DList<StructEl> StructElList;
+struct StructDefList : DList<StructDef> {};
 
 struct TypeMapEl
 	: public AvlTreeEl<TypeMapEl>
@@ -563,22 +598,26 @@ struct TypeMapEl
 	enum Type
 	{
 		AliasType = 1,
-		StructType,
-		LangElType
+		LangElType,
+		StructType
 	};
 
 	const String &getKey() { return key; }
 
 	TypeMapEl( Type type, const String &key, TypeRef *typeRef )
-		: type(type), key(key), value(0), typeRef(typeRef) {}
+		: type(type), key(key), value(0), typeRef(typeRef), structEl(0) {}
 
 	TypeMapEl( Type type, const String &key, LangEl *value )
-		: type(type), key(key), value(value), typeRef(0) {}
+		: type(type), key(key), value(value), typeRef(0), structEl(0) {}
+
+	TypeMapEl( Type type, const String &key, StructEl *structEl )
+		: type(type), key(key), value(0), typeRef(0), structEl(structEl) {}
 
 	Type type;
 	String key;
 	LangEl *value;
 	TypeRef *typeRef;
+	StructEl *structEl;
 	
 	TypeMapEl *prev, *next;
 };
@@ -652,7 +691,7 @@ struct TokenRegion
 
 	TokenRegion *next, *prev;
 
-	/* Tree traversal. */
+	/* tree_t traversal. */
 	void findName( Compiler *pd );
 };
 
@@ -695,41 +734,43 @@ typedef DList<TokenRegion> RegionList;
 typedef DList<RegionImpl> RegionImplList;
 
 typedef Vector<Namespace*> NamespaceVect;
+typedef Vector<Reduction*> ReductionVect;
 
+/* Generics have runtime-representations, so we must track them as unique
+ * types. This gives the runtimes some idea of what is contained in the
+ * structures. */
 struct GenericType 
 	: public DListEl<GenericType>
 {
-	GenericType( const String &name, long typeId, long id, 
-			LangEl *langEl, TypeRef *typeArg )
+	GenericType( long typeId, long id, TypeRef *elTr,
+			TypeRef *keyTr, TypeRef *valueTr, ObjectField *el )
 	:
-		name(name), typeId(typeId), id(id), langEl(langEl),
-		typeArg(typeArg), keyTypeArg(0), 
-		utArg(0), keyUT(0), objDef(0), elOffset(0)
+		typeId(typeId), id(id),
+		elTr(elTr), keyTr(keyTr), valueTr(valueTr),
+		elUt(0), keyUt(0), valueUt(0),
+		objDef(0), el(el), elOffset(0)
 	{}
 
-	const String &getKey() const 
-		{ return name; };
-
 	void declare( Compiler *pd, Namespace *nspace );
 
-	String name;
 	long typeId;
 	long id;
-	LangEl *langEl;
-	TypeRef *typeArg;
-	TypeRef *keyTypeArg;
-	UniqueType *utArg;
-	UniqueType *keyUT;
+
+	TypeRef *elTr;
+	TypeRef *keyTr;
+	TypeRef *valueTr;
+
+	UniqueType *elUt;
+	UniqueType *keyUt;
+	UniqueType *valueUt;
+
 	ObjectDef *objDef;
+	ObjectField *el;
 	long elOffset;
 };
 
 typedef DList<GenericType> GenericList;
 
-typedef struct _UserIter UserIter;
-typedef AvlMap<String, UserIter*, CmpStr> UserIterMap;
-typedef AvlMapEl<String, UserIter*> UserIterMapEl;
-
 /* Graph dictionary. */
 struct GraphDictEl 
 :
@@ -776,16 +817,68 @@ struct TypeAlias
 
 typedef DList<TypeAlias> TypeAliasList;
 
+typedef AvlMap<String, ObjectField*, CmpStr> FieldMap;
+typedef AvlMapEl<String, ObjectField*> FieldMapEl;
+
+typedef AvlMap<String, ObjectMethod*, CmpStr> MethodMap;
+typedef AvlMapEl<String, ObjectMethod*> MethodMapEl;
+
+/* tree_t of name scopes for an object def. All of the object fields inside this
+ * tree live in one object def. This is used for scoping names in functions. */
+struct NameScope
+{
+	NameScope()
+	:
+		owningObj(0),
+		parentScope(0),
+		childIter(0)
+	{}
+
+	ObjectDef *owningObj;
+	FieldMap fieldMap;	
+	MethodMap methodMap;	
+
+	NameScope *parentScope;
+	DList<NameScope> children;
+
+	/* For iteration after declaration. */
+	NameScope *childIter;
+
+	NameScope *prev, *next;
+
+	int depth()
+	{
+		int depth = 0;
+		NameScope *scope = this;
+		while ( scope != 0 ) {
+			depth += 1;
+			scope = scope->parentScope;
+		}
+		return depth;
+	}
+
+	ObjectField *findField( const String &name ) const;
+	ObjectMethod *findMethod( const String &name ) const;
+
+	ObjectField *checkRedecl( const String &name );
+	void insertField( const String &name, ObjectField *value );
+};
+
+
 struct Namespace
 {
 	/* Construct with a list of joins */
 	Namespace( const InputLoc &loc, const String &name, int id, 
 			Namespace *parentNamespace ) : 
 		loc(loc), name(name), id(id),
-		parentNamespace(parentNamespace) { }
+		parentNamespace(parentNamespace)
+	{
+		rootScope = new NameScope;
+	}
 
-	/* Tree traversal. */
+	/* tree_t traversal. */
 	Namespace *findNamespace( const String &name );
+	Reduction *findReduction( const String &name );
 
 	InputLoc loc;
 	String name;
@@ -801,8 +894,7 @@ struct Namespace
 	/* List of nonterminal defs in the namespace. */
 	NtDefList ntDefList;
 
-	/* List of context definitions for encapsulating the data of a parser. */
-	ContextDefList contextDefList;
+	StructDefList structDefList;
 
 	/* Dictionary of symbols within the region. */
 	TypeMap typeMap;
@@ -816,6 +908,10 @@ struct Namespace
 	Namespace *parentNamespace;
 	NamespaceVect childNamespaces;
 
+	ReductionVect reductions;
+
+	NameScope *rootScope;
+
 	Namespace *next, *prev;
 
 	void declare( Compiler *pd );
@@ -824,6 +920,92 @@ struct Namespace
 typedef DList<Namespace> NamespaceList;
 typedef BstSet< Namespace*, CmpOrd<Namespace*> > NamespaceSet;
 
+struct ReduceTextItem
+{
+	enum Type {
+		LhsRef,
+		RhsRef,
+		RhsLoc,
+		Txt
+	};
+
+	ReduceTextItem() : n(0) {}
+
+	Type type;
+	String txt;
+	int n;
+
+	ReduceTextItem *prev, *next;
+};
+
+typedef DList<ReduceTextItem> ReduceTextItemList;
+
+struct ReduceNonTerm
+{
+	ReduceNonTerm( const InputLoc &loc, TypeRef *nonTerm )
+	:
+		loc(loc),
+		nonTerm(nonTerm)
+	{}
+
+	InputLoc loc;
+	TypeRef *nonTerm;
+	ReduceTextItemList itemList;
+
+	ReduceNonTerm *prev, *next;
+};
+
+struct ReduceAction
+{
+	ReduceAction( const InputLoc &loc, TypeRef *nonTerm,
+			const String &prod )
+	:
+		loc(loc), nonTerm(nonTerm),
+		prod(prod),
+		production(0)
+	{}
+
+	InputLoc loc;
+	TypeRef *nonTerm;
+	String prod;
+	ReduceTextItemList itemList;
+
+
+	Production *production;
+
+	ReduceAction *prev, *next;
+};
+
+typedef DList<ReduceAction> ReduceActionList;
+typedef DList<ReduceNonTerm> ReduceNonTermList;
+
+typedef Vector<ReduceAction*> ReduceActionVect;
+
+struct Reduction
+{
+	Reduction( const InputLoc &loc, String name )
+	:
+		loc(loc), name(name),
+		needData(0), needLoc(0)
+	{
+		static int nextId = 1;	
+		id = nextId++;
+		var = name.data;
+		var.data[0] = tolower( var.data[0] );
+	}
+
+	InputLoc loc;
+	String name;
+	String var;
+	int id;
+
+	bool *needData;
+	bool *needLoc;
+
+	ReduceActionList reduceActions;
+	ReduceNonTermList reduceNonTerms;
+};
+
 /*
  * LexJoin
  */
@@ -843,7 +1025,7 @@ struct LexJoin
 		return j;
 	}
 
-	/* Tree traversal. */
+	/* tree_t traversal. */
 	FsmGraph *walk( Compiler *pd );
 	void makeNameTree( Compiler *pd );
 	void varDecl( Compiler *pd, TokenDef *tokenDef );
@@ -902,7 +1084,7 @@ struct LexExpression
 
 	~LexExpression();
 
-	/* Tree traversal. */
+	/* tree_t traversal. */
 	FsmGraph *walk( Compiler *pd, bool lastInSeq = true );
 	void makeNameTree( Compiler *pd );
 	void varDecl( Compiler *pd, TokenDef *tokenDef );
@@ -988,7 +1170,7 @@ struct LexFactorAug
 
 	~LexFactorAug();
 
-	/* Tree traversal. */
+	/* tree_t traversal. */
 	FsmGraph *walk( Compiler *pd );
 	void makeNameTree( Compiler *pd );
 	void varDecl( Compiler *pd, TokenDef *tokenDef );
@@ -1050,7 +1232,7 @@ struct LexFactorRep
 
 	~LexFactorRep();
 
-	/* Tree traversal. */
+	/* tree_t traversal. */
 	FsmGraph *walk( Compiler *pd );
 	void makeNameTree( Compiler *pd );
 
@@ -1100,7 +1282,7 @@ struct LexFactorNeg
 
 	~LexFactorNeg();
 
-	/* Tree traversal. */
+	/* tree_t traversal. */
 	FsmGraph *walk( Compiler *pd );
 	void makeNameTree( Compiler *pd );
 
@@ -1121,7 +1303,7 @@ struct LexFactor
 		OrExprType,
 		RegExprType, 
 		ReferenceType,
-		ParenType,
+		ParenType
 	}; 
 	
 	LexFactor()
@@ -1195,7 +1377,7 @@ struct LexFactor
 	/* Cleanup. */
 	~LexFactor();
 
-	/* Tree traversal. */
+	/* tree_t traversal. */
 	FsmGraph *walk( Compiler *pd );
 	void makeNameTree( Compiler *pd );
 
@@ -1556,7 +1738,8 @@ struct ConsItem
 		expr(0),
 		langEl(0),
 		prodEl(0),
-		bindId(-1)
+		bindId(-1),
+		trim(false)
 	{
 	}
 
@@ -1569,12 +1752,13 @@ struct ConsItem
 		return r;
 	}
 
-	static ConsItem *cons( const InputLoc &loc, Type type, LangExpr *expr )
+	static ConsItem *cons( const InputLoc &loc, Type type, LangExpr *expr, bool trim )
 	{
 		ConsItem *r = new ConsItem;
 		r->loc = loc;
 		r->type = type;
 		r->expr = expr;
+		r->trim = trim;
 		return r;
 	}
 
@@ -1595,6 +1779,7 @@ struct ConsItem
 	LangEl *langEl;
 	ProdEl *prodEl;
 	long bindId;
+	bool trim;
 	ConsItem *prev, *next;
 };
 
@@ -1614,6 +1799,7 @@ struct ConsItemList
 		return new ConsItemList;
 	}
 
+	void resolve( Compiler *pd );
 	void evaluateSendStream( Compiler *pd, CodeVect &code );
 };
 
@@ -1638,7 +1824,7 @@ struct Pattern
 	PatternItemList *list;
 	long patRepId;
 	LangEl *langEl;
-	PdaRun *pdaRun;
+	struct pda_run *pdaRun;
 	long nextBindId;
 	Pattern *prev, *next;
 };
@@ -1667,7 +1853,7 @@ struct Constructor
 	ConsItemList *list;
 	int patRepId;
 	LangEl *langEl;
-	PdaRun *pdaRun;
+	struct pda_run *pdaRun;
 	long nextBindId;
 	bool parse;
 
@@ -1679,7 +1865,8 @@ typedef DList<Constructor> ConsList;
 struct ParserText
 {
 	static ParserText *cons( const InputLoc &loc,
-			Namespace *nspace, ConsItemList *list )
+			Namespace *nspace, ConsItemList *list,
+			bool used, bool reduce, const String &reducer )
 	{
 		ParserText *p = new ParserText;
 		p->loc = loc;
@@ -1689,6 +1876,10 @@ struct ParserText
 		p->pdaRun = 0;
 		p->nextBindId = 1;
 		p->parse = true;
+		p->used = used;
+		p->reduce = reduce;
+		p->reducer = reducer;
+		p->reducerId = -1;
 		return p;
 	}
 
@@ -1696,9 +1887,14 @@ struct ParserText
 	Namespace *nspace;
 	ConsItemList *list;
 	LangEl *langEl;
-	PdaRun *pdaRun;
+	struct pda_run *pdaRun;
 	long nextBindId;
 	bool parse;
+	bool used;
+	bool reduce;
+	String reducer;
+	int reducerId;
+
 	ParserText *prev, *next;
 };
 
@@ -1708,27 +1904,45 @@ struct Function;
 
 struct IterDef
 {
-	enum Type { Tree, Child, RevChild, Repeat, RevRepeat, User };
+	enum Type { Tree, Child, RevChild, Repeat,
+			RevRepeat, User, ListEl, 
+			RevListVal, MapEl };
 
 	IterDef( Type type, Function *func );
 	IterDef( Type type );
 
 	Type type;
 
+	Function *func;
+};
+
+struct IterImpl
+{
+	enum Type { Tree, Child, RevChild, Repeat,
+			RevRepeat, User, ListEl, ListVal,
+			RevListVal, MapEl, MapVal };
+
+	IterImpl( Type type, Function *func );
+	IterImpl( Type type );
+
+	Type type;
+
 	Function *func;
 	bool useFuncId;
 	bool useSearchUT;
+	bool useGenericId;
 
-	Code inCreateWV;
-	Code inCreateWC;
-	Code inDestroy;
-	Code inAdvance;
+	code_t inCreateWV;
+	code_t inCreateWC;
+	code_t inUnwind;
+	code_t inDestroy;
+	code_t inAdvance;
 
-	Code inGetCurR;
-	Code inGetCurWC;
-	Code inSetCurWC;
+	code_t inGetCurR;
+	code_t inGetCurWC;
+	code_t inSetCurWC;
 
-	Code inRefFromCur;
+	code_t inRefFromCur;
 };
 
 struct CmpIterDef
@@ -1795,26 +2009,66 @@ struct UniqueType : public AvlTreeEl<UniqueType>
 	UniqueType( enum TYPE typeId ) :
 		typeId(typeId), 
 		langEl(0), 
-		iterDef(0)
+		iterDef(0),
+		structEl(0),
+		generic(0)
 	{}
 
 	UniqueType( enum TYPE typeId, LangEl *langEl ) :
 		typeId(typeId),
 		langEl(langEl),
-		iterDef(0)
+		iterDef(0),
+		structEl(0),
+		generic(0)
 	{}
 
 	UniqueType( enum TYPE typeId, IterDef *iterDef ) :
 		typeId(typeId),
 		langEl(0),
-		iterDef(iterDef)
+		iterDef(iterDef),
+		structEl(0),
+		generic(0)
+	{}
+
+	UniqueType( enum TYPE typeId, StructEl *structEl ) :
+		typeId(typeId),
+		langEl(0),
+		iterDef(0),
+		structEl(structEl),
+		generic(0)
+	{}
+
+	UniqueType( enum TYPE typeId, GenericType *generic ) :
+		typeId(typeId),
+		langEl(0),
+		iterDef(0),
+		structEl(0),
+		generic(generic)
 	{}
 
 	enum TYPE typeId;
 	LangEl *langEl;
 	IterDef *iterDef;
+	StructEl *structEl;
+	GenericType *generic;
 
 	ObjectDef *objectDef();
+
+	bool tree()
+		{ return typeId == TYPE_TREE; }
+	
+	bool parser()
+		{ return typeId == TYPE_GENERIC && generic->typeId == GEN_PARSER; }
+	
+	bool ptr()
+		{ return typeId == TYPE_STRUCT || typeId == TYPE_GENERIC; }
+
+	bool val() {
+		return typeId == TYPE_STRUCT ||
+			typeId == TYPE_GENERIC ||
+			typeId == TYPE_INT ||
+			typeId == TYPE_BOOL;
+	}
 };
 
 struct CmpUniqueType
@@ -1828,7 +2082,7 @@ enum RepeatType {
 	RepeatNone = 1,
 	RepeatRepeat,
 	RepeatList,
-	RepeatOpt,
+	RepeatOpt
 };
 
 /*
@@ -1855,176 +2109,56 @@ struct CmpUniqueRepeat
 typedef AvlBasic< UniqueRepeat, CmpUniqueRepeat > UniqueRepeatMap;
 
 /* 
- * Unique Map Types
+ * Unique generics. Allows us to do singleton declarations of generic types and
+ * supporting structures. For example, the list type, but also the list element
+ * struct created for the list type.
  */
 
-struct UniqueMap
-	: public AvlTreeEl<UniqueMap>
+struct UniqueGeneric
+	: public AvlTreeEl<UniqueGeneric>
 {
-	UniqueMap( UniqueType *key, UniqueType *value ) :
-		key(key), value(value), generic(0) {}
+	enum Type
+	{
+		List,
+		ListEl,
+		Map,
+		MapEl,
+		Parser
+	};
 
+	UniqueGeneric( Type type, UniqueType *value )
+	:
+		type(type),
+		key(0),
+		value(value),
+		generic(0),
+		structEl(0)
+	{}
+
+	UniqueGeneric( Type type, UniqueType *key, UniqueType *value )
+	:
+		type(type),
+		key(key),
+		value(value),
+		generic(0),
+		structEl(0)
+	{}
+
+	Type type;
 	UniqueType *key;
 	UniqueType *value;
 
 	GenericType *generic;
+	StructEl *structEl;
 };
 
-struct CmpUniqueMap
+struct CmpUniqueGeneric
 {
-	static int compare( const UniqueMap &ut1, const UniqueMap &ut2 );
+	static int compare( const UniqueGeneric &ut1,
+			const UniqueGeneric &ut2 );
 };
 
-typedef AvlBasic< UniqueMap, CmpUniqueMap > UniqueMapMap;
-
-/* 
- * Unique List Types
- */
-
-struct UniqueList
-	: public AvlTreeEl<UniqueList>
-{
-	UniqueList( UniqueType *value ) :
-		value(value), generic(0) {}
-
-	UniqueType *value;
-	GenericType *generic;
-};
-
-struct CmpUniqueList
-{
-	static int compare( const UniqueList &ut1, const UniqueList &ut2 );
-};
-
-typedef AvlBasic< UniqueList, CmpUniqueList > UniqueListMap;
-
-/* 
- * Unique Vector Types
- */
-
-struct UniqueVector
-	: public AvlTreeEl<UniqueVector>
-{
-	UniqueVector( UniqueType *value ) :
-		value(value), generic(0) {}
-
-	UniqueType *value;
-	GenericType *generic;
-};
-
-struct CmpUniqueVector
-{
-	static int compare( const UniqueVector &ut1, const UniqueVector &ut2 );
-};
-
-typedef AvlBasic< UniqueVector, CmpUniqueVector > UniqueVectorMap;
-
-/* 
- * Unique Parser Types
- */
-
-struct UniqueParser
-	: public AvlTreeEl<UniqueParser>
-{
-	UniqueParser( UniqueType *parseType ) :
-		parseType(parseType), generic(0) {}
-
-	UniqueType *parseType;
-	GenericType *generic;
-};
-
-struct CmpUniqueParser
-{
-	static int compare( const UniqueParser &ut1, const UniqueParser &ut2 );
-};
-
-typedef AvlBasic< UniqueParser, CmpUniqueParser > UniqueParserMap;
-
-/* 
- * Unique List2El Types
- */
-
-struct UniqueList2El
-	: public AvlTreeEl<UniqueList2El>
-{
-	UniqueList2El( UniqueType *value ) :
-		value(value), generic(0) {}
-
-	UniqueType *value;
-	GenericType *generic;
-};
-
-struct CmpUniqueList2El
-{
-	static int compare( const UniqueList2El &ut1, const UniqueList2El &ut2 );
-};
-
-typedef AvlBasic< UniqueList2El, CmpUniqueList2El > UniqueList2ElMap;
-
-/* 
- * Unique List2 Types
- */
-
-struct UniqueList2
-	: public AvlTreeEl<UniqueList2>
-{
-	UniqueList2( UniqueType *value, int attrOff ) :
-		value(value), attrOff(attrOff), generic(0) {}
-
-	UniqueType *value;
-	int attrOff;
-	GenericType *generic;
-};
-
-struct CmpUniqueList2
-{
-	static int compare( const UniqueList2 &ut1, const UniqueList2 &ut2 );
-};
-
-typedef AvlBasic< UniqueList2, CmpUniqueList2 > UniqueList2Map;
-
-
-/* 
- * Unique Map2El Types
- */
-
-struct UniqueMap2El
-	: public AvlTreeEl<UniqueMap2El>
-{
-	UniqueMap2El( UniqueType *value ) :
-		value(value), generic(0) {}
-
-	UniqueType *value;
-	GenericType *generic;
-};
-
-struct CmpUniqueMap2El
-{
-	static int compare( const UniqueMap2El &ut1, const UniqueMap2El &ut2 );
-};
-
-typedef AvlBasic< UniqueMap2El, CmpUniqueMap2El > UniqueMap2ElMap;
-
-/* 
- * Unique Map2 Types
- */
-
-struct UniqueMap2
-	: public AvlTreeEl<UniqueMap2>
-{
-	UniqueMap2( UniqueType *value ) :
-		value(value), generic(0) {}
-
-	UniqueType *value;
-	GenericType *generic;
-};
-
-struct CmpUniqueMap2
-{
-	static int compare( const UniqueMap2 &ut1, const UniqueMap2 &ut2 );
-};
-
-typedef AvlBasic< UniqueMap2, CmpUniqueMap2 > UniqueMap2Map;
+typedef AvlBasic< UniqueGeneric, CmpUniqueGeneric > UniqueGenericMap;
 
 /*
  *
@@ -2044,14 +2178,13 @@ struct TypeRef
 		Literal,
 		Iterator,
 		List,
+		ListPtrs,
+		ListEl,
 		Map,
+		MapEl,
+		MapPtrs,
 		Parser,
-		Ref,
-		Ptr,
-		List2El,
-		List2,
-		Map2El,
-		Map2,
+		Ref
 	};
 
 	TypeRef()
@@ -2063,6 +2196,7 @@ struct TypeRef
 		iterDef(0),
 		typeRef1(0),
 		typeRef2(0),
+		typeRef3(0),
 		repeatType(RepeatNone),
 		parsedVarRef(0),
 		parsedTypeRef(0),
@@ -2174,6 +2308,21 @@ struct TypeRef
 		t->repeatType = RepeatNone;
 		return t;
 	}
+
+	static TypeRef *cons( const InputLoc &loc, Type type, 
+			NamespaceQual *nspaceQual, TypeRef *typeRef1,
+			TypeRef *typeRef2, TypeRef *typeRef3 )
+	{
+		TypeRef *t = new TypeRef;
+		t->type = type;
+		t->loc = loc;
+		t->nspaceQual = nspaceQual;
+		t->typeRef1 = typeRef1;
+		t->typeRef2 = typeRef2;
+		t->typeRef3 = typeRef3;
+		t->repeatType = RepeatNone;
+		return t;
+	}
 	
 	/* Pointers and Refs. */
 	static TypeRef *cons( const InputLoc &loc, Type type, TypeRef *typeRef1 )
@@ -2217,24 +2366,19 @@ struct TypeRef
 	UniqueType *resolveIterator( Compiler *pd );
 	UniqueType *resolveTypeName( Compiler *pd );
 	UniqueType *resolveTypeLiteral( Compiler *pd );
-	UniqueType *resolveTypeMapObj( Compiler *pd );
-	UniqueType *resolveTypeMap( Compiler *pd );
-	UniqueType *resolveTypeListObj( Compiler *pd );
 	UniqueType *resolveTypeList( Compiler *pd );
-	UniqueType *resolveTypeParserObj( Compiler *pd );
+	UniqueType *resolveTypeListEl( Compiler *pd );
+	UniqueType *resolveTypeMap( Compiler *pd );
+	UniqueType *resolveTypeMapEl( Compiler *pd );
 	UniqueType *resolveTypeParser( Compiler *pd );
 	UniqueType *resolveType( Compiler *pd );
-	UniqueType *resolveTypePtr( Compiler *pd );
 	UniqueType *resolveTypeRef( Compiler *pd );
 
-	UniqueType *resolveTypeList2ElObj( Compiler *pd );
-	UniqueType *resolveTypeList2El( Compiler *pd );
-	UniqueType *resolveTypeList2Obj( Compiler *pd );
-	UniqueType *resolveTypeList2( Compiler *pd );
-	UniqueType *resolveTypeMap2ElObj( Compiler *pd );
-	UniqueType *resolveTypeMap2El( Compiler *pd );
-	UniqueType *resolveTypeMap2Obj( Compiler *pd );
-	UniqueType *resolveTypeMap2( Compiler *pd );
+	bool uniqueGeneric( UniqueGeneric *&inMap,
+			Compiler *pd, const UniqueGeneric &searchKey );
+
+	StructEl *declareMapElStruct( Compiler *pd, TypeRef *keyType, TypeRef *valType );
+	StructEl *declareListEl( Compiler *pd, TypeRef *valType );
 
 	Type type;
 	InputLoc loc;
@@ -2245,6 +2389,7 @@ struct TypeRef
 	IterDef *iterDef;
 	TypeRef *typeRef1;
 	TypeRef *typeRef2;
+	TypeRef *typeRef3;
 	RepeatType repeatType;
 
 	/* For pattern and constructor context. */
@@ -2280,13 +2425,17 @@ struct ObjectMethod
 		useFuncId(false),
 		useCallObj(true),
 		func(0), 
-		iterDef(0)
+		iterDef(0),
+		useFnInstr(false),
+		useGenericId(false),
+		generic(0)
 	{
 	}
 
 	ObjectMethod( UniqueType *returnUT, String name, 
 			int opcodeWV, int opcodeWC, int numParams, 
-			UniqueType **types, ParameterList *paramList, bool isConst )
+			UniqueType **types, ParameterList *paramList,
+			bool isConst )
 	: 
 		returnUT(returnUT),
 		returnTypeRef(0),
@@ -2301,7 +2450,10 @@ struct ObjectMethod
 		useFuncId(false),
 		useCallObj(true),
 		func(0), 
-		iterDef(0)
+		iterDef(0),
+		useFnInstr(false),
+		useGenericId(false),
+		generic(0)
 	{
 		this->paramUTs = new UniqueType*[numParams];
 		memcpy( this->paramUTs, types, sizeof(UniqueType*)*numParams );
@@ -2322,10 +2474,11 @@ struct ObjectMethod
 	bool useCallObj;
 	Function *func;
 	IterDef *iterDef;
-};
+	bool useFnInstr;
 
-typedef AvlMap<String, ObjectMethod*, CmpStr> MethodMap;
-typedef AvlMapEl<String, ObjectMethod*> MethodMapEl;
+	bool useGenericId;
+	GenericType *generic;
+};
 
 struct RhsVal
 {
@@ -2352,25 +2505,36 @@ struct ObjectField
 		RhsNameType,
 		ParamValType,
 		ParamRefType,
-		LexSubstrType
+		LexSubstrType,
+		GenericElementType,
+		GenericDependentType
 	};
 
 	ObjectField()
 	: 
 		typeRef(0),
-		context(0),
 		scope(0),
 		offset(0),
 		beenReferenced(false),
 		isConst(false), 
 		refActive(false),
 		isExport(false),
+		isConstVal(false),
+		useGenericId(false),
+		generic(0),
+		mapKeyField(0),
 		dirtyTree(false),
 		inGetR( IN_HALT ),
 		inGetWC( IN_HALT ),
 		inGetWV( IN_HALT ),
 		inSetWC( IN_HALT ),
-		inSetWV( IN_HALT )
+		inSetWV( IN_HALT ),
+		inGetValR( IN_HALT ),
+		inGetValWC( IN_HALT ),
+		inGetValWV( IN_HALT ),
+		inSetValWC( IN_HALT ),
+		inSetValWV( IN_HALT ),
+		iterImpl( 0 )
 	{}
 
 	static ObjectField *cons( const InputLoc &loc,
@@ -2405,18 +2569,44 @@ struct ObjectField
 
 	bool isInbuiltObject()
 		{ return type == InbuiltObjectType; }
+
+	bool exists()
+	{
+		switch ( type ) {
+			case ObjectField::LhsElType:
+			case ObjectField::UserLocalType:
+			case ObjectField::RedRhsType:
+			case ObjectField::UserFieldType:
+			case ObjectField::StructFieldType:
+			case ObjectField::GenericDependentType:
+				return true;
+			default:
+				return false;
+		}
+	}
 	
 	InputLoc loc;
 	Type type;
 	TypeRef *typeRef;
 	String name;
-	Context *context;
 	NameScope *scope;
 	long offset;
 	bool beenReferenced;
+	/* Declared const. */
 	bool isConst;
 	bool refActive;
 	bool isExport;
+
+	/* Value is a const thing when that retrieved by the runtime. Requires a
+	 * const val id. */
+	bool isConstVal;
+	int constValId;
+	String constValArg;
+
+	bool useGenericId;
+	GenericType *generic;
+
+	ObjectField *mapKeyField;
 	
 	/* True if some aspect of the tree has possibly been written to. This does
 	 * not include attributes. This is here so we can optimize the storage of
@@ -2426,62 +2616,26 @@ struct ObjectField
 
 	Vector<RhsVal> rhsVal;
 
-	Code inGetR;
-	Code inGetWC;
-	Code inGetWV;
-	Code inSetWC;
-	Code inSetWV;
+	code_t inGetR;
+	code_t inGetWC;
+	code_t inGetWV;
+	code_t inSetWC;
+	code_t inSetWV;
+	code_t inGetValR;
+	code_t inGetValWC;
+	code_t inGetValWV;
+	code_t inSetValWC;
+	code_t inSetValWV;
+
+	IterImpl *iterImpl;
 
 	ObjectField *prev, *next;
 };
 
-typedef AvlMap<String, ObjectField*, CmpStr> FieldMap;
-typedef AvlMapEl<String, ObjectField*> FieldMapEl;
-
 typedef DListVal<ObjectField*> FieldList;
 
 typedef DList<ObjectField> ParameterList; 
 
-struct TemplateType;
-
-/* Tree of name scopes for an object def. All of the object fields inside this
- * tree live in one object def. This is used for scoping names in functions. */
-struct NameScope
-{
-	NameScope()
-	:
-		owner(0),
-		parentScope(0),
-		childIter(0)
-	{}
-
-	ObjectDef *owner;
-	FieldMap *fieldMap;	
-
-	NameScope *parentScope;
-	DList<NameScope> children;
-
-	/* For iteration after declaration. */
-	NameScope *childIter;
-
-	NameScope *prev, *next;
-
-	int depth()
-	{
-		int depth = 0;
-		NameScope *scope = this;
-		while ( scope != 0 ) {
-			depth += 1;
-			scope = scope->parentScope;
-		}
-		return depth;
-	}
-
-	ObjectField *findField( const String &name ) const;
-
-	ObjectField *checkRedecl( const String &name );
-	void insertField( const String &name, ObjectField *value );
-};
 
 struct ObjectDef
 {
@@ -2508,19 +2662,14 @@ struct ObjectDef
 		o->id = id;
 
 		o->rootScope = new NameScope;
-		o->rootScope->owner = o;
-		o->rootScope->fieldMap = new FieldMap;
-
-		o->fieldList = new FieldList;
-		o->methodMap = new MethodMap;
+		o->rootScope->owningObj = o;
 
 		return o;
 	}
 
 	Type type;
 	String name;
-	FieldList *fieldList;
-	MethodMap *methodMap;	
+	FieldList fieldList;
 
 	NameScope *rootScope;
 
@@ -2530,16 +2679,15 @@ struct ObjectDef
 	long nextOffset;
 	long firstNonTree;
 
-
 	void referenceField( Compiler *pd, ObjectField *field );
 	void placeField( Compiler *pd, ObjectField *field );
 	void createCode( Compiler *pd, CodeVect &code );
-	ObjectMethod *findMethod( const String &name ) const;
 	ObjectField *findFieldInScope( const NameScope *scope, const String &name ) const;
 	ObjectField *checkRedecl( NameScope *inScope, const String &name );
 	void insertField( NameScope *inScope, const String &name, ObjectField *value );
 	void resolve( Compiler *pd );
 	ObjectField *findFieldNum( long offset );
+	ObjectField *findFieldType( Compiler *pd, UniqueType *ut );
 
 	long size() { return nextOffset; }
 	long sizeTrees() { return firstNonTree; }
@@ -2621,22 +2769,27 @@ typedef Vector<QualItem> QualItemVect;
 
 struct LangVarRef
 {
-	static LangVarRef *cons( const InputLoc &loc, Context *context,
-			NameScope *scope, QualItemVect *qual, const String &name )
+	static LangVarRef *cons( const InputLoc &loc, Namespace *nspace,
+			StructDef *structDef, NameScope *scope,
+			NamespaceQual *nspaceQual, QualItemVect *qual,
+			const String &name )
 	{
 		LangVarRef *l = new LangVarRef;
 		l->loc = loc;
-		l->context = context;
+		l->nspace = nspace;
+		l->structDef = structDef;
 		l->scope = scope;
+		l->nspaceQual = nspaceQual;
 		l->qual = qual;
 		l->name = name;
 		return l;
 	}
 
-	static LangVarRef *cons( const InputLoc &loc, Context *context,
-			NameScope *scope, const String &name )
+	static LangVarRef *cons( const InputLoc &loc, Namespace *nspace,
+			StructDef *structDef, NameScope *scope, const String &name )
 	{
-		return cons( loc, context, scope, new QualItemVect, name );
+		return cons( loc, nspace, structDef, scope,
+				NamespaceQual::cons( nspace ), new QualItemVect, name );
 	}
 
 	void resolve( Compiler *pd ) const;
@@ -2645,15 +2798,17 @@ struct LangVarRef
 	UniqueType *loadField( Compiler *pd, CodeVect &code, ObjectDef *inObject,
 			ObjectField *el, bool forWriting, bool revert ) const;
 
+	VarRefLookup lookupIterCall( Compiler *pd ) const;
 	VarRefLookup lookupMethod( Compiler *pd ) const;
 	VarRefLookup lookupField( Compiler *pd ) const;
 
 	VarRefLookup lookupQualification( Compiler *pd, NameScope *rootScope ) const;
 	VarRefLookup lookupObj( Compiler *pd ) const;
+	VarRefLookup lookupMethodObj( Compiler *pd ) const;
 
 	bool isInbuiltObject() const;
 	bool isLocalRef() const;
-	bool isContextRef() const;
+	bool isStructRef() const;
 	void loadQualification( Compiler *pd, CodeVect &code, NameScope *rootScope, 
 			int lastPtrInQual, bool forWriting, bool revert ) const;
 	void loadInbuiltObject( Compiler *pd, CodeVect &code, 
@@ -2665,8 +2820,11 @@ struct LangVarRef
 	void loadGlobalObj( Compiler *pd, CodeVect &code, 
 			int lastPtrInQual, bool forWriting ) const;
 	void loadObj( Compiler *pd, CodeVect &code, int lastPtrInQual, bool forWriting ) const;
-	void canTakeRef( Compiler *pd, VarRefLookup &lookup ) const;
-	bool canTakeRefTest( Compiler *pd, VarRefLookup &lookup ) const;
+	void loadScopedObj( Compiler *pd, CodeVect &code, 
+		NameScope *scope, int lastPtrInQual, bool forWriting ) const;
+
+	void verifyRefPossible( Compiler *pd, VarRefLookup &lookup ) const;
+	bool canTakeRef( Compiler *pd, VarRefLookup &lookup ) const;
 
 	void setFieldIter( Compiler *pd, CodeVect &code, ObjectDef *inObject,
 			ObjectField *objField, UniqueType *objUT, UniqueType *exprType,
@@ -2678,9 +2836,11 @@ struct LangVarRef
 
 	void assignValue( Compiler *pd, CodeVect &code, UniqueType *exprUT ) const;
 
+	IterImpl *chooseTriterCall( Compiler *pd, UniqueType *searchUT, CallArgVect *args );
+
 	/* The deref generics value is for iterator calls with lists and maps as args. */
 	ObjectField **evaluateArgs( Compiler *pd, CodeVect &code, 
-			VarRefLookup &lookup, CallArgVect *args, bool derefGenerics = false );
+			VarRefLookup &lookup, CallArgVect *args );
 
 	void callOperation( Compiler *pd, CodeVect &code, VarRefLookup &lookup ) const;
 	UniqueType *evaluateCall( Compiler *pd, CodeVect &code, CallArgVect *args );
@@ -2692,9 +2852,12 @@ struct LangVarRef
 	void popRefQuals( Compiler *pd, CodeVect &code, 
 			VarRefLookup &lookup, CallArgVect *args, bool temps ) const;
 
+
 	InputLoc loc;
-	Context *context;
+	Namespace *nspace;
+	StructDef *structDef;
 	NameScope *scope;
+	NamespaceQual *nspaceQual;
 	QualItemVect *qual;
 	String name;
 	long argSize;
@@ -2723,12 +2886,14 @@ struct LangTerm
 		MakeTreeType,
 		MakeTokenType,
 		EmbedStringType,
-		CastType,
+		CastType
 	};
 
 	LangTerm()
 	:
 		generic(0),
+		constructor(0),
+		consItemList(0),
 		parserText(0)
 	{}
 
@@ -2841,7 +3006,7 @@ struct LangTerm
 
 	static LangTerm *cons( const InputLoc &loc, Type type, LangVarRef *varRef,
 			ObjectField *objField, TypeRef *typeRef, FieldInitVect *fieldInitArgs,
-			Constructor *constructor, ParserText *parserText )
+			ConsItemList *consItemList, ParserText *parserText )
 	{
 		LangTerm *t = new LangTerm;
 		t->loc = loc;
@@ -2850,7 +3015,7 @@ struct LangTerm
 		t->objField = objField;
 		t->typeRef = typeRef;
 		t->fieldInitArgs = fieldInitArgs;
-		t->constructor = constructor;
+		t->consItemList = consItemList;
 		t->parserText = parserText;
 		return t;
 	}
@@ -2907,12 +3072,26 @@ struct LangTerm
 		s->eof = eof;
 		return s;
 	}
+
+	static LangTerm *consNew( const InputLoc &loc, TypeRef *typeRef,
+			LangVarRef *captureVarRef, FieldInitVect *fieldInitArgs )
+	{
+		LangTerm *s = new LangTerm;
+		s->type = NewType;
+		s->loc = loc;
+		s->typeRef = typeRef;
+		s->varRef = captureVarRef;
+		s->fieldInitArgs = fieldInitArgs;
+		return s;
+	}
 	
+	void resolveFieldArgs( Compiler *pd );
 	void resolve( Compiler *pd );
 
+	void evaluateCapture( Compiler *pd, CodeVect &code, UniqueType *valUt ) const;
+	void evaluateCapture( Compiler *pd, CodeVect &code, bool isTree ) const;
 	UniqueType *evaluateNew( Compiler *pd, CodeVect &code ) const;
 	UniqueType *evaluateConstruct( Compiler *pd, CodeVect &code ) const;
-	UniqueType *evaluateNewstruct( Compiler *pd, CodeVect &code ) const;
 	void parseFrag( Compiler *pd, CodeVect &code, int stopId ) const;
 	UniqueType *evaluateParse( Compiler *pd, CodeVect &code, bool tree, bool stop ) const;
 	void evaluateSendStream( Compiler *pd, CodeVect &code ) const;
@@ -2927,6 +3106,7 @@ struct LangTerm
 	UniqueType *evaluateEmbedString( Compiler *pd, CodeVect &code ) const;
 	UniqueType *evaluateSearch( Compiler *pd, CodeVect &code ) const;
 	UniqueType *evaluateCast( Compiler *pd, CodeVect &code ) const;
+	void resolveFieldArgs( Compiler *pd ) const;
 
 	InputLoc loc;
 	Type type;
@@ -2940,9 +3120,9 @@ struct LangTerm
 	FieldInitVect *fieldInitArgs;
 	GenericType *generic;
 	Constructor *constructor;
+	ConsItemList *consItemList;
 	ParserText *parserText;
 	LangExpr *expr;
-	ConsItemList *consItemList;
 	bool eof;
 };
 
@@ -2988,7 +3168,7 @@ struct LangExpr
 	void resolve( Compiler *pd ) const;
 
 	UniqueType *evaluate( Compiler *pd, CodeVect &code ) const;
-	bool canTakeRefTest( Compiler *pd ) const;
+	bool canTakeRef( Compiler *pd ) const;
 
 	InputLoc loc;
 	Type type;
@@ -3199,7 +3379,7 @@ struct LangStmt
 
 	static LangStmt *cons( const InputLoc &loc, Type type, ObjectField *objField,
 			TypeRef *typeRef, IterCall *iterCall, StmtList *stmtList,
-			Context *context, NameScope *scope )
+			StructDef *context, NameScope *scope )
 	{
 		LangStmt *s = new LangStmt;
 		s->loc = loc;
@@ -3259,7 +3439,7 @@ struct LangStmt
 	LangStmt *elsePart;
 	String name;
 	IterCall *iterCall;
-	Context *context;
+	StructDef *context;
 	NameScope *scope;
 	ConsItemList *consItemList;
 
@@ -3293,7 +3473,7 @@ struct CodeBlock
 	StmtList *stmtList;
 	ObjectDef *localFrame;
 	Locals locals;
-	Context *context;
+	StructDef *context;
 
 	/* Each frame has two versions of 
 	 * the code: revert and commit. */
@@ -3304,18 +3484,21 @@ struct Function
 {
 	Function()
 	:
+		nspace(0),
 		paramListSize(0),
 		paramUTs(0),
 		inContext(0),
-		objMethod(0)
+		objMethod(0),
+		inHost(false)
 	{}
 
-	static Function *cons( TypeRef *typeRef, const String &name, 
+	static Function *cons( Namespace *nspace, TypeRef *typeRef, const String &name, 
 			ParameterList *paramList, CodeBlock *codeBlock, 
 			int funcId, bool isUserIter, bool exprt )
 	{
 		Function *f = new Function;
 
+		f->nspace = nspace;
 		f->typeRef = typeRef;
 		f->name = name;
 		f->paramList = paramList;
@@ -3327,9 +3510,11 @@ struct Function
 		return f;
 	}
 
+	Namespace *nspace;
 	TransBlock *transBlock;
 	TypeRef *typeRef;
 	String name;
+	String hostCall;
 	ParameterList *paramList;
 	CodeBlock *codeBlock;
 	ObjectDef *localFrame;
@@ -3337,13 +3522,15 @@ struct Function
 	bool isUserIter;
 	long paramListSize;
 	UniqueType **paramUTs;
-	Context *inContext;
+	StructDef *inContext;
 	bool exprt;
 	ObjectMethod *objMethod;
+	bool inHost;
 
 	Function *prev, *next;
 };
 
 typedef DList<Function> FunctionList;
 
-#endif /* _PARSETREE_H */
+#endif /* _COLM_PARSETREE_H */
+
