@@ -1,12 +1,35 @@
 /*
- *  Copyright 2006-2012 Adrian Thurston <thurston@complang.org>
+ * Copyright 2006-2012 Adrian Thurston <thurston@colm.net>
+ *
+ * Permission is hereby granted, free of charge, to any person obtaining a copy
+ * of this software and associated documentation files (the "Software"), to
+ * deal in the Software without restriction, including without limitation the
+ * rights to use, copy, modify, merge, publish, distribute, sublicense, and/or
+ * sell copies of the Software, and to permit persons to whom the Software is
+ * furnished to do so, subject to the following conditions:
+ *
+ * The above copyright notice and this permission notice shall be included in all
+ * copies or substantial portions of the Software.
+ *
+ * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
+ * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
+ * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
+ * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
+ * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
+ * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
+ * SOFTWARE.
  */
 
-#include <iostream>
-#include <sstream>
 #include "redfsm.h"
-#include "avlmap.h"
-#include "mergesort.h"
+
+#include <assert.h>
+#include <stdbool.h>
+#include <string.h>
+#include <stdio.h>
+
+#include <sstream>
+#include <iostream>
+
 #include "fsmgraph.h"
 #include "parsetree.h"
 
@@ -812,25 +835,25 @@ int eofAction( RedState *state )
 }
 
 
-FsmTables *RedFsm::makeFsmTables()
+fsm_tables *RedFsm::makeFsmTables()
 {
 	/* The fsm runtime needs states sorted by id. */
 	sortByStateId();
 
 	int pos, curKeyOffset, curIndOffset;
-	FsmTables *fsmTables = new FsmTables;
-	fsmTables->numStates = stateList.length();
+	fsm_tables *fsmTables = new fsm_tables;
+	fsmTables->num_states = stateList.length();
 
 	/*
 	 * actions
 	 */
 
-	fsmTables->numActions = 1;
+	fsmTables->num_actions = 1;
 	for ( GenActionTableMap::Iter act = actionMap; act.lte(); act++ )
-		fsmTables->numActions += 1 + act->key.length();
+		fsmTables->num_actions += 1 + act->key.length();
 
 	pos = 0;
-	fsmTables->actions = new long[fsmTables->numActions];
+	fsmTables->actions = new long[fsmTables->num_actions];
 	fsmTables->actions[pos++] = 0;
 	for ( GenActionTableMap::Iter act = actionMap; act.lte(); act++ ) {
 		fsmTables->actions[pos++] = act->key.length();
@@ -842,10 +865,10 @@ FsmTables *RedFsm::makeFsmTables()
 	 * keyOffset
 	 */
 	pos = 0, curKeyOffset = 0;
-	fsmTables->keyOffsets = new long[fsmTables->numStates];
+	fsmTables->key_offsets = new long[fsmTables->num_states];
 	for ( RedStateList::Iter st = stateList; st.lte(); st++ ) {
 		/* Store the current offset. */
-		fsmTables->keyOffsets[pos++] = curKeyOffset;
+		fsmTables->key_offsets[pos++] = curKeyOffset;
 
 		/* Move the key offset ahead. */
 		curKeyOffset += st->outSingle.length() + st->outRange.length()*2;
@@ -854,20 +877,20 @@ FsmTables *RedFsm::makeFsmTables()
 	/*
 	 * transKeys
 	 */
-	fsmTables->numTransKeys = 0;
+	fsmTables->num_trans_keys = 0;
 	for ( RedStateList::Iter st = stateList; st.lte(); st++ ) {
-		fsmTables->numTransKeys += st->outSingle.length();
-		fsmTables->numTransKeys += 2 * st->outRange.length();
+		fsmTables->num_trans_keys += st->outSingle.length();
+		fsmTables->num_trans_keys += 2 * st->outRange.length();
 	}
 
 	pos = 0;
-	fsmTables->transKeys = new char[fsmTables->numTransKeys];
+	fsmTables->trans_keys = new char[fsmTables->num_trans_keys];
 	for ( RedStateList::Iter st = stateList; st.lte(); st++ ) {
 		for ( RedTransList::Iter stel = st->outSingle; stel.lte(); stel++ )
-			fsmTables->transKeys[pos++] = stel->lowKey.getVal();
+			fsmTables->trans_keys[pos++] = stel->lowKey.getVal();
 		for ( RedTransList::Iter rtel = st->outRange; rtel.lte(); rtel++ ) {
-			fsmTables->transKeys[pos++] = rtel->lowKey.getVal();
-			fsmTables->transKeys[pos++] = rtel->highKey.getVal();
+			fsmTables->trans_keys[pos++] = rtel->lowKey.getVal();
+			fsmTables->trans_keys[pos++] = rtel->highKey.getVal();
 		}
 	}
 
@@ -875,25 +898,25 @@ FsmTables *RedFsm::makeFsmTables()
 	 * singleLengths
 	 */
 	pos = 0;
-	fsmTables->singleLengths = new long[fsmTables->numStates];
+	fsmTables->single_lengths = new long[fsmTables->num_states];
 	for ( RedStateList::Iter st = stateList; st.lte(); st++ )
-		fsmTables->singleLengths[pos++] = st->outSingle.length();
+		fsmTables->single_lengths[pos++] = st->outSingle.length();
 
 	/*
 	 * rangeLengths
 	 */
 	pos = 0;
-	fsmTables->rangeLengths = new long[fsmTables->numStates];
+	fsmTables->range_lengths = new long[fsmTables->num_states];
 	for ( RedStateList::Iter st = stateList; st.lte(); st++ )
-		fsmTables->rangeLengths[pos++] = st->outRange.length();
+		fsmTables->range_lengths[pos++] = st->outRange.length();
 
 	/*
 	 * indexOffsets
 	 */
 	pos = 0, curIndOffset = 0;
-	fsmTables->indexOffsets = new long[fsmTables->numStates];
+	fsmTables->index_offsets = new long[fsmTables->num_states];
 	for ( RedStateList::Iter st = stateList; st.lte(); st++ ) {
-		fsmTables->indexOffsets[pos++] = curIndOffset;
+		fsmTables->index_offsets[pos++] = curIndOffset;
 
 		curIndOffset += st->outSingle.length() + st->outRange.length();
 		if ( st->defTrans != 0 )
@@ -952,63 +975,63 @@ FsmTables *RedFsm::makeFsmTables()
 	 * toStateActions
 	 */
 	pos = 0;
-	fsmTables->toStateActions = new long[fsmTables->numStates];
+	fsmTables->to_state_actions = new long[fsmTables->num_states];
 	for ( RedStateList::Iter st = stateList; st.lte(); st++ )
-		fsmTables->toStateActions[pos++] = toStateAction( st );
+		fsmTables->to_state_actions[pos++] = toStateAction( st );
 
 	/*
 	 * fromStateActions
 	 */
 	pos = 0;
-	fsmTables->fromStateActions = new long[fsmTables->numStates];
+	fsmTables->from_state_actions = new long[fsmTables->num_states];
 	for ( RedStateList::Iter st = stateList; st.lte(); st++ )
-		fsmTables->fromStateActions[pos++] = fromStateAction( st );
+		fsmTables->from_state_actions[pos++] = fromStateAction( st );
 
 	/*
 	 * eofActions
 	 */
 	pos = 0;
-	fsmTables->eofActions = new long[fsmTables->numStates];
+	fsmTables->eof_actions = new long[fsmTables->num_states];
 	for ( RedStateList::Iter st = stateList; st.lte(); st++ )
-		fsmTables->eofActions[pos++] = eofAction( st );
+		fsmTables->eof_actions[pos++] = eofAction( st );
 
 	/*
 	 * eofTargs
 	 */
 	pos = 0;
-	fsmTables->eofTargs = new long[fsmTables->numStates];
+	fsmTables->eof_targs = new long[fsmTables->num_states];
 	for ( RedStateList::Iter st = stateList; st.lte(); st++ ) {
 		int targ = -1;
 		if ( st->eofTrans != 0 )
 			targ = st->eofTrans->targ->id;
-		fsmTables->eofTargs[pos++] = targ;
+		fsmTables->eof_targs[pos++] = targ;
 	}
 
 	/* Start state. */
-	fsmTables->startState = startState->id;
+	fsmTables->start_state = startState->id;
 
 	/* First final state. */
-	fsmTables->firstFinal = ( firstFinState != 0 ) ?
+	fsmTables->first_final = ( firstFinState != 0 ) ?
 		firstFinState->id : nextStateId;
 
 	/* The error state. */
-	fsmTables->errorState = ( errState != 0 ) ?
+	fsmTables->error_state = ( errState != 0 ) ?
 		errState->id : -1;
 
 	/* The array pointing to actions. */
 	pos = 0;
-	fsmTables->numActionSwitch = genActionList.length();
-	fsmTables->actionSwitch = new GenAction*[fsmTables->numActionSwitch];
+	fsmTables->num_action_switch = genActionList.length();
+	fsmTables->action_switch = new GenAction*[fsmTables->num_action_switch];
 	for ( GenActionList::Iter act = genActionList; act.lte(); act++ )
-		fsmTables->actionSwitch[pos++] = act;
+		fsmTables->action_switch[pos++] = act;
 	
 	/*
 	 * entryByRegion
 	 */
 
-	fsmTables->numRegions = regionToEntry.length()+1;
-	fsmTables->entryByRegion = new long[fsmTables->numRegions];
-	fsmTables->entryByRegion[0] = fsmTables->errorState;
+	fsmTables->num_regions = regionToEntry.length()+1;
+	fsmTables->entry_by_region = new long[fsmTables->num_regions];
+	fsmTables->entry_by_region[0] = fsmTables->error_state;
 
 	pos = 1;
 	for ( RegionToEntry::Iter en = regionToEntry; en.lte(); en++ ) {
@@ -1016,8 +1039,8 @@ FsmTables *RedFsm::makeFsmTables()
 		RedEntryMapEl *entryMapEl = redEntryMap.find( *en );
 		
 		/* Save it off. */
-		fsmTables->entryByRegion[pos++] = entryMapEl != 0 ? entryMapEl->value 
-				: fsmTables->errorState;
+		fsmTables->entry_by_region[pos++] = entryMapEl != 0 ? entryMapEl->value 
+				: fsmTables->error_state;
 	}
 	
 	return fsmTables;
