@@ -1,11 +1,33 @@
 /*
- *  Copyright 2009-2012 Adrian Thurston <thurston@complang.org>
+ * Copyright 2009-2012 Adrian Thurston <thurston@colm.net>
+ *
+ * Permission is hereby granted, free of charge, to any person obtaining a copy
+ * of this software and associated documentation files (the "Software"), to
+ * deal in the Software without restriction, including without limitation the
+ * rights to use, copy, modify, merge, publish, distribute, sublicense, and/or
+ * sell copies of the Software, and to permit persons to whom the Software is
+ * furnished to do so, subject to the following conditions:
+ *
+ * The above copyright notice and this permission notice shall be included in all
+ * copies or substantial portions of the Software.
+ *
+ * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
+ * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
+ * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
+ * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
+ * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
+ * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
+ * SOFTWARE.
  */
 
-#include "bytecode.h"
-#include "compiler.h"
+#include <stdbool.h>
+
 #include <iostream>
-#include <assert.h>
+
+#include "compiler.h"
+/*
+ * Type Resolution.
+ */
 
 using std::cout;
 using std::cerr;
@@ -46,15 +68,12 @@ UniqueType *TypeRef::resolveTypeName( Compiler *pd )
 					return inDict->typeRef->resolveType( pd );
 				}
 
-				case TypeMapEl::StructType: {
-					UniqueType *structUt = pd->findUniqueType( TYPE_TREE, inDict->value );
-					return pd->findUniqueType( TYPE_PTR, structUt->langEl );
-				}
-
 				case TypeMapEl::LangElType: {
 					UniqueType *ut = pd->findUniqueType( TYPE_TREE, inDict->value );
-					if ( ut == pd->uniqueTypeStream )
-						return pd->findUniqueType( TYPE_PTR, ut->langEl );
+					return ut;
+				}
+				case TypeMapEl::StructType: {
+					UniqueType *ut = pd->findUniqueType( TYPE_STRUCT, inDict->structEl );
 					return ut;
 				}
 			}
@@ -66,7 +85,7 @@ UniqueType *TypeRef::resolveTypeName( Compiler *pd )
 		nspace = nspace->parentNamespace;
 	}
 
-	error(loc) << "unknown type in typeof expression" << endp;
+	error(loc) << "unknown type " << typeName << " in typeof expression" << endp;
 	return 0;
 }
 
@@ -96,66 +115,108 @@ UniqueType *TypeRef::resolveTypeLiteral( Compiler *pd )
 		nspace = nspace->parentNamespace;
 	}
 
-	error(loc) << "unknown type in typeof expression" << endp;
+	error(loc) << "unknown type " << typeName << " in typeof expression" << endp;
 	return 0;
 }
 
-UniqueType *TypeRef::resolveTypeMapObj( Compiler *pd )
+bool TypeRef::uniqueGeneric( UniqueGeneric *&inMap, Compiler *pd,
+		const UniqueGeneric &searchKey )
 {
-	nspace = pd->rootNamespace;
-
-	UniqueType *utKey = typeRef1->resolveType( pd );	
-	UniqueType *utValue = typeRef2->resolveType( pd );	
-
-	UniqueMap searchKey( utKey, utValue );
-	UniqueMap *inMap = pd->uniqueMapMap.find( &searchKey );
+	bool inserted = false;
+	inMap = pd->uniqueGenericMap.find( &searchKey );
 	if ( inMap == 0 ) {
-		inMap = new UniqueMap( utKey, utValue );
-		pd->uniqueMapMap.insert( inMap );
-
-		/* FIXME: Need uniqe name allocator for types. */
-		static int mapId = 0;
-		String name( 36, "__map%d", mapId++ );
-
-		GenericType *generic = new GenericType( name, GEN_MAP,
-				pd->nextGenericId++, 0/*langEl*/, typeRef2 );
-		generic->keyTypeArg = typeRef1;
-
-		nspace->genericList.append( generic );
-
-		generic->declare( pd, nspace );
-
-		inMap->generic = generic;
+		inserted = true;
+		inMap = new UniqueGeneric( searchKey );
+		pd->uniqueGenericMap.insert( inMap );
 	}
-
-	generic = inMap->generic;
-	return pd->findUniqueType( TYPE_TREE, inMap->generic->langEl );
+	return inserted;
 }
 
-UniqueType *TypeRef::resolveTypeMap( Compiler *pd )
+StructEl *TypeRef::declareListEl( Compiler *pd, TypeRef *valType )
 {
-	UniqueType *mapUt = resolveTypeMapObj( pd );
-	return pd->findUniqueType( TYPE_PTR, mapUt->langEl );
+	static long vlistElId = 1;
+	String name( 32, "list_el_%d", vlistElId++ );
+	ObjectDef *objectDef = ObjectDef::cons( ObjectDef::StructType,
+			name, pd->nextObjectId++ );
+
+	StructDef *structDef = new StructDef( loc, name, objectDef );
+
+	pd->rootNamespace->structDefList.append( structDef );
+
+	/* Value Element. */
+	String id = "value";
+	ObjectField *elValObjField = ObjectField::cons( internal,
+				ObjectField::StructFieldType, valType, id );
+
+	objectDef->rootScope->insertField( elValObjField->name, elValObjField );
+
+	/* Typeref for the struct. Used for pointers. */
+	NamespaceQual *nspaceQual = NamespaceQual::cons( pd->rootNamespace );
+	TypeRef *selfTypeRef = TypeRef::cons( InputLoc(), nspaceQual, name, RepeatNone );
+
+	/* Type ref for the list pointers psuedo type. */
+	TypeRef *elTr = TypeRef::cons( InputLoc(), TypeRef::ListPtrs, 0, selfTypeRef, 0 );
+
+	ObjectField *of = ObjectField::cons( InputLoc(),
+			ObjectField::GenericElementType, elTr, name );
+
+	objectDef->rootScope->insertField( of->name, of );
+
+	return declareStruct( pd, pd->rootNamespace, name, structDef );
 }
 
-UniqueType *TypeRef::resolveTypeListObj( Compiler *pd )
+void ConsItemList::resolve( Compiler *pd )
+{
+	/* Types in constructor. */
+	for ( ConsItemList::Iter item = first(); item.lte(); item++ ) {
+		switch ( item->type ) {
+		case ConsItem::LiteralType:
+			/* Use pdaFactor reference resolving. */
+			pd->resolveProdEl( item->prodEl );
+			break;
+		case ConsItem::InputText:
+			break;
+		case ConsItem::ExprType:
+			item->expr->resolve( pd );
+			break;
+		}
+	}
+}
+
+UniqueType *TypeRef::resolveTypeListEl( Compiler *pd )
+{
+	TypeRef *valTr = typeRef1;
+	UniqueType *utValue = valTr->resolveType( pd );	
+
+	UniqueGeneric *inMap = 0, searchKey( UniqueGeneric::ListEl, utValue );
+	if ( uniqueGeneric( inMap, pd, searchKey ) )
+		inMap->structEl = declareListEl( pd, valTr );
+
+	return pd->findUniqueType( TYPE_STRUCT, inMap->structEl );
+}
+
+UniqueType *TypeRef::resolveTypeList( Compiler *pd )
 {
 	nspace = pd->rootNamespace;
 
 	UniqueType *utValue = typeRef1->resolveType( pd );	
 
-	UniqueList searchKey( utValue );
-	UniqueList *inMap = pd->uniqueListMap.find( &searchKey );
-	if ( inMap == 0 ) {
-		inMap = new UniqueList( utValue );
-		pd->uniqueListMap.insert( inMap );
+	if ( utValue->typeId != TYPE_STRUCT )
+		error( loc ) << "only structs can be list elements" << endp;
 
-		/* FIXME: Need uniqe name allocator for types. */
-		static int listId = 0;
-		String name( 36, "__list%d", listId++ );
+	/* Find the list element. */
+	ObjectDef *elObjDef = utValue->structEl->structDef->objectDef;
+	UniqueType *ptrsUt = pd->findUniqueType( TYPE_LIST_PTRS );
+	ObjectField *listEl = elObjDef->findFieldType( pd, ptrsUt );
 
-		GenericType *generic = new GenericType( name, GEN_LIST,
-				pd->nextGenericId++, 0/*langEl*/, typeRef1 );
+	if ( !listEl )
+		error( loc ) << "could not find list element in type ref" << endp;
+
+	UniqueGeneric *inMap = 0, searchKey( UniqueGeneric::List, utValue );
+	if ( uniqueGeneric( inMap, pd, searchKey ) ) {
+
+		GenericType *generic = new GenericType( GEN_LIST,
+				pd->nextGenericId++, typeRef1, 0, typeRef2, listEl );
 
 		nspace->genericList.append( generic );
 
@@ -165,33 +226,104 @@ UniqueType *TypeRef::resolveTypeListObj( Compiler *pd )
 	}
 
 	generic = inMap->generic;
-	return pd->findUniqueType( TYPE_TREE, inMap->generic->langEl );
+	return pd->findUniqueType( TYPE_GENERIC, inMap->generic );
 }
 
-UniqueType *TypeRef::resolveTypeList( Compiler *pd )
+StructEl *TypeRef::declareMapElStruct( Compiler *pd, TypeRef *keyType, TypeRef *valType )
 {
-	UniqueType *listUt = resolveTypeListObj( pd );
-	return pd->findUniqueType( TYPE_PTR, listUt->langEl );
+	static long vlistElId = 1;
+	String name( 32, "map_el_%d", vlistElId++ );
+	ObjectDef *objectDef = ObjectDef::cons( ObjectDef::StructType,
+			name, pd->nextObjectId++ );
+
+	StructDef *structDef = new StructDef( loc, name, objectDef );
+
+	pd->rootNamespace->structDefList.append( structDef );
+
+	/* Value Element. */
+	String id = "value";
+	ObjectField *elValObjField = ObjectField::cons( internal,
+				ObjectField::StructFieldType, valType, id );
+
+	objectDef->rootScope->insertField( elValObjField->name, elValObjField );
+
+	/* Typeref for the pointers. */
+	NamespaceQual *nspaceQual = NamespaceQual::cons( pd->rootNamespace );
+	TypeRef *selfTypeRef = TypeRef::cons( InputLoc(), nspaceQual, name, RepeatNone );
+
+	TypeRef *elTr = TypeRef::cons( InputLoc(), TypeRef::MapPtrs, 0, selfTypeRef, keyType );
+
+	ObjectField *of = ObjectField::cons( InputLoc(),
+			ObjectField::GenericElementType, elTr, name );
+
+	objectDef->rootScope->insertField( of->name, of );
+
+	StructEl *sel = declareStruct( pd, pd->rootNamespace, name, structDef );
+	return sel;
 }
 
-UniqueType *TypeRef::resolveTypeParserObj( Compiler *pd )
+UniqueType *TypeRef::resolveTypeMapEl( Compiler *pd )
+{
+	TypeRef *keyType = typeRef1;
+	TypeRef *valType = typeRef2;
+
+	UniqueType *utKey = keyType->resolveType( pd );	
+	UniqueType *utValue = valType->resolveType( pd );	
+
+	UniqueGeneric *inMap = 0, searchKey( UniqueGeneric::MapEl, utKey, utValue );
+	if ( uniqueGeneric( inMap, pd, searchKey ) )
+		inMap->structEl = declareMapElStruct( pd, keyType, valType );
+
+	return pd->findUniqueType( TYPE_STRUCT, inMap->structEl );
+}
+
+
+UniqueType *TypeRef::resolveTypeMap( Compiler *pd )
+{
+	nspace = pd->rootNamespace;
+
+	UniqueType *utKey = typeRef1->resolveType( pd );	
+	UniqueType *utEl = typeRef2->resolveType( pd );	
+
+	if ( utEl->typeId != TYPE_STRUCT )
+		error( loc ) << "only structs can be map elements" << endp;
+
+	/* Find the list element. */
+	ObjectDef *elObjDef = utEl->structEl->structDef->objectDef;
+	UniqueType *ptrsUt = pd->findUniqueType( TYPE_MAP_PTRS );
+	ObjectField *mapEl = elObjDef->findFieldType( pd, ptrsUt );
+
+	if ( !mapEl )
+		error( loc ) << "could not find map element in type ref" << endp;
+
+	UniqueGeneric *inMap = 0, searchKey( UniqueGeneric::Map, utKey, utEl );
+
+	if ( uniqueGeneric( inMap, pd, searchKey ) ) {
+
+		GenericType *generic = new GenericType( GEN_MAP,
+				pd->nextGenericId++, typeRef2, typeRef1, typeRef3, mapEl );
+
+		nspace->genericList.append( generic );
+
+		generic->declare( pd, nspace );
+
+		inMap->generic = generic;
+	}
+
+	generic = inMap->generic;
+	return pd->findUniqueType( TYPE_GENERIC, inMap->generic );
+}
+
+UniqueType *TypeRef::resolveTypeParser( Compiler *pd )
 {
 	nspace = pd->rootNamespace;
 
 	UniqueType *utParse = typeRef1->resolveType( pd );	
 
-	UniqueParser searchKey( utParse );
-	UniqueParser *inMap = pd->uniqueParserMap.find( &searchKey );
-	if ( inMap == 0 ) {
-		inMap = new UniqueParser( utParse );
-		pd->uniqueParserMap.insert( inMap );
-
-		/* FIXME: Need uniqe name allocator for types. */
-		static int accumId = 0;
-		String name( 36, "__accum%d", accumId++ );
-
-		GenericType *generic = new GenericType( name, GEN_PARSER,
-				pd->nextGenericId++, 0/*langEl*/, typeRef1 );
+	UniqueGeneric *inMap = 0, searchKey( UniqueGeneric::Parser, utParse );
+	if ( uniqueGeneric( inMap, pd, searchKey ) ) {
+		GenericType *generic = new GenericType( GEN_PARSER,
+				pd->nextGenericId++, typeRef1, 0, 0, 0 );
 
 		nspace->genericList.append( generic );
 
@@ -201,187 +333,14 @@ UniqueType *TypeRef::resolveTypeParserObj( Compiler *pd )
 	}
 
 	generic = inMap->generic;
-	return pd->findUniqueType( TYPE_TREE, inMap->generic->langEl );
+	return pd->findUniqueType( TYPE_GENERIC, inMap->generic );
 }
 
-UniqueType *TypeRef::resolveTypeParser( Compiler *pd )
-{
-	UniqueType *parserUt = resolveTypeParserObj( pd );
-	return pd->findUniqueType( TYPE_PTR, parserUt->langEl );
-}
-
-/*
- * Object-based list/map
- */
-UniqueType *TypeRef::resolveTypeList2ElObj( Compiler *pd )
-{
-	nspace = pd->rootNamespace;
-
-	UniqueType *utValue = typeRef1->resolveType( pd );	
-
-	UniqueList2El searchKey( utValue );
-	UniqueList2El *inMap = pd->uniqueList2ElMap.find( &searchKey );
-	if ( inMap == 0 ) {
-		inMap = new UniqueList2El( utValue );
-		pd->uniqueList2ElMap.insert( inMap );
-
-		/* FIXME: Need uniqe name allocator for types. */
-		static int listId = 0;
-		String name( 36, "__list2el%d", listId++ );
-
-		GenericType *generic = new GenericType( name, GEN_LIST2EL,
-				pd->nextGenericId++, 0/*langEl*/, typeRef1 );
-
-		nspace->genericList.append( generic );
-
-		generic->declare( pd, nspace );
-
-		inMap->generic = generic;
-	}
-
-	generic = inMap->generic;
-	return pd->findUniqueType( TYPE_TREE, inMap->generic->langEl );
-}
-
-UniqueType *TypeRef::resolveTypeList2El( Compiler *pd )
-{
-	UniqueType *listUt = resolveTypeList2ElObj( pd );
-	return pd->findUniqueType( TYPE_PTR, listUt->langEl );
-}
-
-UniqueType *TypeRef::resolveTypeList2Obj( Compiler *pd )
-{
-	nspace = pd->rootNamespace;
-
-	UniqueType *utValue = typeRef1->resolveType( pd );	
-
-	/* Find the offset of the list element. */
-	int off = 0;
-	bool found = false;
-	FieldList *fieldList = utValue->langEl->objectDef->fieldList;
-	for ( FieldList::Iter f = *fieldList; f.lte(); f++, off++ ) {
-		UniqueType *fUT = f->value->typeRef->resolveType( pd );
-		if ( fUT->langEl->generic != 0 &&
-				fUT->langEl->generic->typeId == GEN_LIST2EL )
-		{
-			found = true;
-			break;
-		}
-	}
-
-	if ( !found )
-		error( loc ) << "cound not find list element in type ref" << endp;
-
-	UniqueList2 searchKey( utValue, off );
-	UniqueList2 *inMap = pd->uniqueList2Map.find( &searchKey );
-	if ( inMap == 0 ) {
-		inMap = new UniqueList2( utValue, off );
-		pd->uniqueList2Map.insert( inMap );
-
-		/* FIXME: Need uniqe name allocator for types. */
-		static int listId = 0;
-		String name( 36, "__list2%d", listId++ );
-
-		GenericType *generic = new GenericType( name, GEN_LIST2,
-				pd->nextGenericId++, 0/*langEl*/, typeRef1 );
-
-		generic->elOffset = off;
-		nspace->genericList.append( generic );
-		generic->declare( pd, nspace );
-		inMap->generic = generic;
-	}
-
-	generic = inMap->generic;
-	return pd->findUniqueType( TYPE_TREE, inMap->generic->langEl );
-}
-
-UniqueType *TypeRef::resolveTypeList2( Compiler *pd )
-{
-	UniqueType *listUt = resolveTypeList2Obj( pd );
-	return pd->findUniqueType( TYPE_PTR, listUt->langEl );
-}
-
-UniqueType *TypeRef::resolveTypeMap2ElObj( Compiler *pd )
-{
-	nspace = pd->rootNamespace;
-
-	UniqueType *utValue = typeRef1->resolveType( pd );	
-
-	UniqueMap2El searchKey( utValue );
-	UniqueMap2El *inMap = pd->uniqueMap2ElMap.find( &searchKey );
-	if ( inMap == 0 ) {
-		inMap = new UniqueMap2El( utValue );
-		pd->uniqueMap2ElMap.insert( inMap );
-
-		/* FIXME: Need uniqe name allocator for types. */
-		static int listId = 0;
-		String name( 36, "__map2el%d", listId++ );
-
-		GenericType *generic = new GenericType( name, GEN_MAP2EL,
-				pd->nextGenericId++, 0/*langEl*/, typeRef1 );
-
-		nspace->genericList.append( generic );
-
-		generic->declare( pd, nspace );
-
-		inMap->generic = generic;
-	}
-
-	generic = inMap->generic;
-	return pd->findUniqueType( TYPE_TREE, inMap->generic->langEl );
-}
-
-UniqueType *TypeRef::resolveTypeMap2El( Compiler *pd )
-{
-	UniqueType *listUt = resolveTypeMap2ElObj( pd );
-	return pd->findUniqueType( TYPE_PTR, listUt->langEl );
-}
-
-UniqueType *TypeRef::resolveTypeMap2Obj( Compiler *pd )
-{
-	nspace = pd->rootNamespace;
-
-	UniqueType *utValue = typeRef1->resolveType( pd );	
-
-	UniqueMap2 searchKey( utValue );
-	UniqueMap2 *inMap = pd->uniqueMap2Map.find( &searchKey );
-	if ( inMap == 0 ) {
-		inMap = new UniqueMap2( utValue );
-		pd->uniqueMap2Map.insert( inMap );
-
-		/* FIXME: Need uniqe name allocator for types. */
-		static int listId = 0;
-		String name( 36, "__map2%d", listId++ );
-
-		GenericType *generic = new GenericType( name, GEN_MAP2,
-				pd->nextGenericId++, 0/*langEl*/, typeRef1 );
-
-		nspace->genericList.append( generic );
-
-		generic->declare( pd, nspace );
-
-		inMap->generic = generic;
-	}
-
-	generic = inMap->generic;
-	return pd->findUniqueType( TYPE_TREE, inMap->generic->langEl );
-}
-
-UniqueType *TypeRef::resolveTypeMap2( Compiler *pd )
-{
-	UniqueType *listUt = resolveTypeMap2Obj( pd );
-	return pd->findUniqueType( TYPE_PTR, listUt->langEl );
-}
 
 /*
  * End object based list/map
  */
 
-UniqueType *TypeRef::resolveTypePtr( Compiler *pd )
-{
-	typeRef1->resolveType( pd );
-	return pd->findUniqueType( TYPE_PTR, typeRef1->uniqueType->langEl );
-}
 
 UniqueType *TypeRef::resolveTypeRef( Compiler *pd )
 {
@@ -441,7 +400,7 @@ UniqueType *TypeRef::resolveIterator( Compiler *pd )
 	UniqueType *searchUT = searchTypeRef->resolveType( pd );
 
 	/* Lookup the iterator call. Make sure it is an iterator. */
-	VarRefLookup lookup = iterCall->langTerm->varRef->lookupMethod( pd );
+	VarRefLookup lookup = iterCall->langTerm->varRef->lookupIterCall( pd );
 	if ( lookup.objMethod->iterDef == 0 ) {
 		error(loc) << "attempt to iterate using something "
 				"that is not an iterator" << endp;
@@ -471,36 +430,36 @@ UniqueType *TypeRef::resolveType( Compiler *pd )
 		case Literal:
 			uniqueType = resolveTypeLiteral( pd );
 			break;
-		case Map:
-			uniqueType = resolveTypeMap( pd );
-			break;
-		case List:
-			uniqueType = resolveTypeList( pd );
-			break;
 		case Parser:
 			uniqueType = resolveTypeParser( pd );
 			break;
-		case Ptr:
-			uniqueType = resolveTypePtr( pd );
-			break;
 		case Ref:
 			uniqueType = resolveTypeRef( pd );
 			break;
 		case Iterator:
 			uniqueType = resolveIterator( pd );
 			break;
-		case List2El:
-			uniqueType = resolveTypeList2El( pd );
+
+		case List:
+			uniqueType = resolveTypeList( pd );
 			break;
-		case List2:
-			uniqueType = resolveTypeList2( pd );
+		case ListPtrs:
+			uniqueType = pd->findUniqueType( TYPE_LIST_PTRS );
 			break;
-		case Map2El:
-			uniqueType = resolveTypeMap2El( pd );
+		case ListEl:
+			uniqueType = resolveTypeListEl( pd );
 			break;
-		case Map2:
-			uniqueType = resolveTypeMap2( pd );
+
+		case Map:
+			uniqueType = resolveTypeMap( pd );
 			break;
+		case MapPtrs:
+			uniqueType = pd->findUniqueType( TYPE_MAP_PTRS );
+			break;
+		case MapEl:
+			uniqueType = resolveTypeMapEl( pd );
+			break;
+			
 		case Unspecified:
 			/* No lookup needed, unique type(s) set when constructed. */
 			break;
@@ -518,31 +477,27 @@ void Compiler::resolveProdEl( ProdEl *prodEl )
 	prodEl->langEl = prodEl->typeRef->uniqueType->langEl;
 }
 
+void LangTerm::resolveFieldArgs( Compiler *pd )
+{
+	/* Initialization expressions. */
+	if ( fieldInitArgs != 0 ) {
+		for ( FieldInitVect::Iter pi = *fieldInitArgs; pi.lte(); pi++ )
+			(*pi)->expr->resolve( pd );
+	}
+}
+
 void LangTerm::resolve( Compiler *pd )
 {
 	switch ( type ) {
 		case ConstructType:
 			typeRef->resolveType( pd );
 
-			/* Initialization expressions. */
-			if ( fieldInitArgs != 0 ) {
-				for ( FieldInitVect::Iter pi = *fieldInitArgs; pi.lte(); pi++ )
-					(*pi)->expr->resolve( pd );
-			}
+			resolveFieldArgs( pd );
 
 			/* Types in constructor. */
-			for ( ConsItemList::Iter item = *constructor->list; item.lte(); item++ ) {
-				switch ( item->type ) {
-				case ConsItem::LiteralType:
-					/* Use pdaFactor reference resolving. */
-					pd->resolveProdEl( item->prodEl );
-					break;
-				case ConsItem::InputText:
-				case ConsItem::ExprType:
-					break;
-				}
-			}
+			constructor->list->resolve( pd );
 			break;
+
 		case VarRefType:
 			break;
 
@@ -574,6 +529,8 @@ void LangTerm::resolve( Compiler *pd )
 
 			break;
 		case NewType:
+			/* Init args, then the new type. */
+			resolveFieldArgs( pd );
 			typeRef->resolveType( pd );
 			break;
 		case TypeIdType:
@@ -591,22 +548,10 @@ void LangTerm::resolve( Compiler *pd )
 		case ParseTreeType:
 		case ParseStopType:
 			typeRef->resolveType( pd );
-			/* Evaluate the initialization expressions. */
-			if ( fieldInitArgs != 0 ) {
-				for ( FieldInitVect::Iter pi = *fieldInitArgs; pi.lte(); pi++ )
-					(*pi)->expr->resolve( pd );
-			}
 
-			for ( ConsItemList::Iter item = *parserText->list; item.lte(); item++ ) {
-				switch ( item->type ) {
-				case ConsItem::LiteralType:
-					pd->resolveProdEl( item->prodEl );
-					break;
-				case ConsItem::InputText:
-				case ConsItem::ExprType:
-					break;
-				}
-			}
+			resolveFieldArgs( pd );
+
+			parserText->list->resolve( pd );
 			break;
 
 		case SendType:
@@ -683,8 +628,10 @@ void LangStmt::resolve( Compiler *pd ) const
 				(*pex)->expr->resolve( pd );
 			break;
 		}
-		case PrintAccum:
+		case PrintAccum: {
+			consItemList->resolve( pd );
 			break;
+		}
 		case ExprType: {
 			/* Evaluate the exrepssion, then pop it immediately. */
 			expr->resolve( pd );
@@ -745,7 +692,7 @@ void LangStmt::resolve( Compiler *pd ) const
 
 void ObjectDef::resolve( Compiler *pd )
 {
-	for ( FieldList::Iter fli = *fieldList; fli.lte(); fli++ ) {
+	for ( FieldList::Iter fli = fieldList; fli.lte(); fli++ ) {
 		ObjectField *field = fli->value;
 
 		if ( field->typeRef != 0 )
@@ -775,6 +722,16 @@ void Compiler::resolveFunction( Function *func )
 	block->resolve( this );
 }
 
+void Compiler::resolveInHost( Function *func )
+{
+	if ( func->typeRef != 0 ) 
+		func->typeRef->resolveType( this );
+
+	for ( ParameterList::Iter param = *func->paramList; param.lte(); param++ )
+		param->typeRef->resolveType( this );
+}
+
+
 void Compiler::resolvePreEof( TokenRegion *region )
 {
 	CodeBlock *block = region->preEofBlock;
@@ -805,6 +762,9 @@ void Compiler::resolveParseTree()
 	for ( FunctionList::Iter f = functionList; f.lte(); f++ )
 		resolveFunction( f );
 
+	for ( FunctionList::Iter f = inHostList; f.lte(); f++ )
+		resolveInHost( f );
+
 	/* Compile the reduction code. */
 	for ( DefList::Iter prod = prodList; prod.lte(); prod++ ) {
 		if ( prod->redBlock != 0 )
@@ -833,17 +793,21 @@ void Compiler::resolveParseTree()
 		ObjectDef *objDef = lel->objectDef;
 		if ( objDef != 0 ) {
 			/* Init all fields of the object. */
-			for ( FieldList::Iter f = *objDef->fieldList; f.lte(); f++ ) {
+			for ( FieldList::Iter f = objDef->fieldList; f.lte(); f++ )
 				f->value->typeRef->resolveType( this );
-			}
 		}
 	}
 
-	/* Init all fields of the global object. */
-	for ( FieldList::Iter f = *globalObjectDef->fieldList; f.lte(); f++ ) {
-		f->value->typeRef->resolveType( this );
+	for ( StructElList::Iter sel = structEls; sel.lte(); sel++ ) {
+		ObjectDef *objDef = sel->structDef->objectDef;
+		for ( FieldList::Iter f = objDef->fieldList; f.lte(); f++ )
+			f->value->typeRef->resolveType( this );
 	}
 
+	/* Init all fields of the global object. */
+	for ( FieldList::Iter f = globalObjectDef->fieldList; f.lte(); f++ ) {
+		f->value->typeRef->resolveType( this );
+	}
 }
 
 /* Resolves production els and computes the precedence of each prod. */
@@ -870,18 +834,6 @@ void Compiler::resolveProductionEls()
 	}
 }
 
-void Compiler::resolveGenericTypes()
-{
-	for ( NamespaceList::Iter ns = namespaceList; ns.lte(); ns++ ) {
-		for ( GenericList::Iter gen = ns->genericList; gen.lte(); gen++ ) {
-			gen->utArg = gen->typeArg->resolveType( this );
-
-			if ( gen->typeId == GEN_MAP )
-				gen->keyUT = gen->keyTypeArg->resolveType( this );
-		}
-	}
-}
-
 void Compiler::makeTerminalWrappers()
 {
 	/* Make terminal language elements corresponding to each nonterminal in
@@ -937,6 +889,56 @@ void Compiler::resolvePrecedence()
 	}
 }
 
+void Compiler::resolveReductionActions()
+{
+	for ( ReductionVect::Iter r = rootNamespace->reductions; r.lte(); r++ ) {
+		for ( ReduceNonTermList::Iter rni = (*r)->reduceNonTerms; rni.lte(); rni++ )
+			rni->nonTerm->resolveType( this );
+
+		for ( ReduceActionList::Iter rai = (*r)->reduceActions; rai.lte(); rai++ )
+			rai->nonTerm->resolveType( this );
+	}
+}
+
+void Compiler::findReductionActionProds()
+{
+	for ( ReductionVect::Iter r = rootNamespace->reductions; r.lte(); r++ ) {
+		for ( ReduceActionList::Iter rai = (*r)->reduceActions; rai.lte(); rai++ ) {
+			rai->nonTerm->resolveType( this );
+			LangEl *langEl = rai->nonTerm->uniqueType->langEl;
+
+			Production *prod = 0;
+			for ( LelDefList::Iter ldi = langEl->defList; ldi.lte(); ldi++ ) {
+				if ( strcmp( ldi->name, rai->prod ) == 0 ) {
+					prod = ldi;
+					break;
+				}
+			}
+
+			if ( prod == 0 ) {
+				error(rai->loc) << "could not find production \"" <<
+						rai->prod << "\"" << endp;
+			}
+
+			rai->production = prod;
+		}
+	}
+}
+
+void Compiler::resolveReducers()
+{
+	for ( ParserTextList::Iter pt = parserTextList; pt.lte(); pt++ ) {
+		if ( pt->reduce ) {
+			Reduction *reduction = rootNamespace->findReduction( pt->reducer );
+			if ( reduction == 0 ) {
+				error ( pt->loc ) << "could not locate reduction \"" <<
+					pt->reducer << "\"" << endp;
+			}
+
+			pt->reducerId = reduction->id;
+		}
+	}
+}
 
 void Compiler::resolvePass()
 {
@@ -948,13 +950,18 @@ void Compiler::resolvePass()
 
 	resolveParseTree();
 
-	resolveGenericTypes();
+	UniqueType *argvUT = argvTypeRef->resolveType( this );
+	argvElSel = argvUT->generic->elUt->structEl;
 
-	argvTypeRef->resolveType( this );
+	resolveReductionActions();
 
 	/* We must do this as the last step in the type resolution process because
 	 * all type resolves can cause new language elments with associated
 	 * productions. They get tacked onto the end of the list of productions.
 	 * Doing it at the end results processing a growing list. */
 	resolveProductionEls();
+
+	findReductionActionProds();
+
+	resolveReducers();
 }
